/*
 * (C) 1997,2003  Jens Ch. Restemeier <jrestemeier@currantbun.com>
 *
 * (C) 2008       Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: afs-save.c,v 1.6 2008/06/11 17:51:34 tneuer Exp $
 *
 *  $Log: afs-save.c,v $
 *  Revision 1.6  2008/06/11 17:51:34  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.5  2008/06/07 17:21:28  tneuer
 *  Using g_fprintf () instead of fprintf () to avoid UTF-8 pitfalls
 *
 *  Revision 1.4  2008/06/07 16:35:58  tneuer
 *  Using g_strerror () instead of strerror ()
 *
 *  Revision 1.3  2008/06/05 15:23:36  tneuer
 *  Use g_fopen () for UTF-8 compatibility
 *
 *  Revision 1.2  2008/05/21 12:30:27  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:05  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/25 12:44:48  tneuer
 *  Initial revision
 *
 */

#include "plugin-conf.h"

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include <glib/gstdio.h>

#include "afs.h"
#include "guf.h"
#include "userfilter.h"


/*
 *  Export the internal filter representation to a
 *  Filter Factory source file.
 *
 *  Values unknown to Filter Factory are not saved
 *  (like version information, tooltips, etc...).
 */

gboolean export_afs (const gchar * const filename,
		     UserFilter *        filter)
{
  FILE * f;
  gint   i;

  if (guf_filter_get_protocol (filter) > GUF_LEVEL_1)
    {
      g_warning ("AFS %s '%s': %s\n",
		 _("Error saving"), filename, _("Protocol level too high."));
      return FALSE;
    }

  if ( (f = g_fopen (filename, "w")) == NULL )
    {
      g_warning ("AFS %s '%s': %s\n",
		 _("Error saving"), filename, g_strerror(errno));
      return FALSE;
    }

  /*
   *  Write header 
   */
  g_fprintf (f, "%%RGB-1.0\r");

  /*
   *  Write sliders 
   */
  for (i = 0; i < N_CONTROLS; i++)
    {
      g_fprintf (f, "%i\r", guf_filter_get_ctl_preset (filter, i));
    }

  /*
   *  Write channels
   */
  for (i = 0; i < N_CHANNELS; i++)
    {
      gchar * src;

      src = guf_filter_get_src (filter, i);
      g_fprintf (f, "%s\r\r", src);
      g_free (src);
    }

  fclose (f);
  return TRUE;
}
