/*
 * (C) 2008  Torsten Neuer <tneuer@inwise.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: checksum.c,v 1.6 2008/06/04 22:44:49 tneuer Exp $
 *
 *  $Log: checksum.c,v $
 *  Revision 1.6  2008/06/04 22:44:49  tneuer
 *  Bugfix: typo in local variable declaration for glib-2.16 checksum routine
 *
 *  Revision 1.5  2008/06/04 10:10:31  tneuer
 *  Bugfix:  missing includes in Glib 2.16+ version
 *
 *  Revision 1.4  2008/05/29 11:44:43  tneuer
 *  Use MD5 from Glib instead of ssl where available
 *
 *  Revision 1.3  2008/05/26 15:10:54  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.2  2008/05/21 12:30:29  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:05  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/10 13:53:42  tneuer
 *  Initial revision
 *
 */

#include "plugin-conf.h"

#include "checksum.h"

#if HAVE_GLIB_2_16

#include <glib.h>
#include <string.h>


guint32
compute_drawable_checksum (GimpDrawable * drawable)
{
  GChecksum    * checksum;
  GimpPixelRgn   rgn;
  gpointer       r;
  guint          v;

  checksum = g_checksum_new (G_CHECKSUM_MD5);

  gimp_pixel_rgn_init (&rgn, drawable, 0, 0,
		       drawable->width, drawable->height, FALSE, FALSE);
  for (r = gimp_pixel_rgns_register (1, &rgn);
       r != NULL;
       r = gimp_pixel_rgns_process (r))
    {
      g_checksum_update (checksum, rgn.data, rgn.w * rgn.h * rgn.bpp);
    }

  memcpy (&v, g_checksum_get_string (checksum), sizeof(v));

  g_checksum_free (checksum);

  return v;
}
#else
#include <openssl/evp.h>

typedef union DIGEST_BUFFER
{
  guchar  md[EVP_MAX_MD_SIZE];
  guint32 val;
} DigestBuffer;


guint32
compute_drawable_checksum (GimpDrawable * drawable)
{
  DigestBuffer result;
  EVP_MD_CTX   checksum;
  GimpPixelRgn rgn;
  gpointer     r;
  guint        l;

  EVP_DigestInit (&checksum, EVP_md5 ());

  gimp_pixel_rgn_init (&rgn, drawable, 0, 0,
		       drawable->width, drawable->height, FALSE, FALSE);
  for (r = gimp_pixel_rgns_register (1, &rgn);
       r != NULL;
       r = gimp_pixel_rgns_process (r))
    {
      EVP_DigestUpdate (&checksum, rgn.data, rgn.w * rgn.h * rgn.bpp);
    }

  EVP_DigestFinal (&checksum, result.md, &l);

  return result.val;
}
#endif
