/*
 * (C) 1997,2003  Jens Ch. Restemeier <jrestemeier@currantbun.com>
 *
 * (C) 2008       Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: ffl.h,v 1.3 2008/06/11 17:51:46 tneuer Exp $
 *
 *  $Log: ffl.h,v $
 *  Revision 1.3  2008/06/11 17:51:46  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.2  2008/05/21 12:30:30  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:02  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/22 15:38:44  tneuer
 *  Initial revision
 *
 */

#ifndef __FFL_H__
#define __FFL_H__

#include "plugin-conf.h"

#include <stdio.h>

#include "guf.h"


typedef struct FFL_INFO
{
  FILE * f;			/* file pointer of library file */
  guint  nfilters;		/* number of filters in library */
} FFL_info;


extern const gchar * const EXT_FFL;

/*
 *  Open library file (s).
 *
 *  Return library file information or NULL.
 */

FFL_info *
ffl_open (const gchar * const filename);


/*
 *  Read one library module (data) from library (f)
 *  at file position (fpos).
 *
 *  If (filename) is not NULL, the internal filename of the
 *  library filter module will be copied to it.
 *
 *  Return TRUE on success, FALSE on failure.
 */

gboolean
ffl_read (FFL_info *   ffl,
	  long         fpos,
	  UserFilter * filter,
	  gchar *      filename);


/*
 *  Close library file (f).
 */

gint
ffl_close (FFL_info * ffl);


#endif /* __FFL_H__ */
