/*
 * (C) 1997,2003  Jens Ch. Restemeier <jrestemeier@currantbun.com>
 *
 * (C) 2008       Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: fileio.c,v 1.6 2008/06/11 17:51:49 tneuer Exp $
 *
 *  $Log: fileio.c,v $
 *  Revision 1.6  2008/06/11 17:51:49  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.5  2008/06/07 17:02:23  tneuer
 *  Using g_ascii_strcasecmp () instead of strcasecmp ()
 *
 *  Revision 1.4  2008/05/26 13:03:11  tneuer
 *  Only display random seed reset widget if filter uses rnd() function
 *
 *  Revision 1.3  2008/05/26 12:01:36  tneuer
 *  Bugfix: editor panel load function has case sensitive fileextension match
 *  Bugfix: last filter filename is not invalidated if load fails
 *
 *  Revision 1.2  2008/05/21 12:30:30  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:03  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/02 11:12:39  tneuer
 *  Initial revision
 *
 */

#include "plugin-conf.h"

#include <string.h>

#include "fileio.h"
#include "guf.h"
#include "txt.h"
#include "afs.h"
#include "8bf.h"


/*
 *  Load a filter from file.
 *
 *  The filter can either be a .guf formatted GKeyFile or
 *  a Filter Factory / PluginCommander style filter (.8bf,
 *  .afs, or .txt).
 */

gboolean
load_filter (const gchar * const filename,
	     UserFilter *        filter)
{
  gchar * x;

  if ((x = strrchr (filename, '.')) != NULL)
    {
      if (!g_ascii_strcasecmp (x, EXT_GUF))
	{
	  return guf_load (filename, GUF_PROTOCOL, filter);
	}
      if (!g_ascii_strcasecmp (x, EXT_TXT))
	{
	  return import_txt (filename, filter);
	}
      if (!g_ascii_strcasecmp (x, EXT_AFS))
	{
	  return import_afs (filename, filter);
	}
      if (!g_ascii_strcasecmp (x, EXT_8BF))
	{
	  return import_8bf (filename, filter);
	}
    }

  return FALSE;
}


/*
 *  Save the internal representation toa file.
 *
 *  This will save either to the standard .guf GKeyFile
 *  formatted filter file or export to a Filter Factory
 *  .afs or PluginCommander .txt.
 */

gboolean
save_filter (const gchar * const filename,
	     UserFilter *        filter)
{
  static const gchar const * guf_comment =
    " Created with User Filter v" PACKAGE_VERSION ".";

  gboolean r;
  gchar *  x;

  if ( (x = strrchr (filename, '.')) != NULL )
    {
      if (!g_ascii_strcasecmp (x, EXT_AFS))
	{
	  return export_afs (filename, filter);
	}
      if (!g_ascii_strcasecmp (x, EXT_TXT))
	{
	  return export_txt (filename, filter);
	}
      if (!g_ascii_strcasecmp (x, EXT_GUF))
	{
	  return guf_save (filename, guf_comment, GUF_PROTOCOL, filter);
	}
    }
  x = g_strdup_printf ("%s%s", filename, EXT_GUF);
  r = guf_save (x, guf_comment, GUF_PROTOCOL, filter);
  g_free (x);
  return r;
}
