/*
 * (C) 2008       Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: guess-year.c,v 1.2 2008/06/11 17:51:51 tneuer Exp $
 *
 *  $Log: guess-year.c,v $
 *  Revision 1.2  2008/06/11 17:51:51  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.1  2008/06/06 22:43:11  tneuer
 *  Add support for "Gimp::Date" entry in 8bf/afs/ffl/txt file import routines
 *
 */

#include "plugin-conf.h"

#include <stdlib.h>

#include <glib.h>
#include <glib/gstdio.h>

#include "guess-year.h"


/*
 *  Guess the year of the date of a filter.
 *
 *  Most copyright information will also contain a year, so we will
 *  try to look it up there.  If this fails, we will take the file
 *  modification time - if this fails, too (which is unlikely), we
 *  fall back to the current system time.
 *  We assume the year 1994 to be the first valid year for a filter
 *  definition file that gets imported for this is the release year
 *  of Filter Factory...
 */

static guint
copyright_year (const gchar * const copyright)
{
  gchar * p;
  guint   y = 0;

  p = (gchar *)copyright;
  while (*p && y < 1994)
    {
      while (*p && !g_ascii_isdigit(*p))
	++p;
      if (*p)
	{
	  y = strtol (p, &p, 10);
	}
    }
  return (y >= 1994) ? y : 0;
}


static guint
filename_year (const gchar * const filename)
{
  struct stat filestat;

  if (!g_stat (filename, &filestat))
    {
      GDate * d;
      guint   y;

      d = g_date_new ();
      g_date_set_time_t (d, filestat.st_mtime);
      y = g_date_get_year (d);
      g_date_free (d);
      return y;
    }
  return 0;
}


guint
guess_year (const gchar * copyright,
	    const gchar * filename)
{
  guint y;

  if (!(y = copyright_year (copyright)) &&
      !(y = filename_year (filename)))
    {
      GDate * d;

      d = g_date_new ();
      g_date_set_time_t (d, time (NULL));
      y = g_date_get_year (d);
      g_date_free (d);
    }
  return y;
}
