/*
 * (C) 2008  Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: guf-read.c,v 1.3 2008/06/11 17:52:03 tneuer Exp $
 *
 *  $Log: guf-read.c,v $
 *  Revision 1.3  2008/06/11 17:52:03  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.2  2008/05/26 15:10:54  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:05  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/22 17:35:48  tneuer
 *  Initial revision
 *
 */

#include "plugin-conf.h"

#include <string.h>
#include <glib.h>

#include "guf.h"


gboolean
guf_read ( const gchar * const filename,
	   const GufProtocol   protocol,
	   UserFilter *        filter )
{
  GError * err = NULL;

  if (g_key_file_load_from_file ((GKeyFile *)filter, filename, GUF_FLAGS, &err))
    {
      if (guf_filter_get_protocol (filter) > protocol)
	{
	  g_warning ("GUF %s '%s': %s\n",
		     _("Error loading"), filename, _("Invalid protocol"));
	  return FALSE;
	}

      return TRUE;
    }
  if (err != NULL)
    {
      g_warning ("GUF %s '%s': %s\n",
		 _("Error loading"), filename, err->message);
      g_error_free (err);
    }
  else
    {
      g_warning ("GUF %s '%s'\n",
		 _("Error loading"), filename);
    }
  return FALSE;
}


gboolean
guf_load ( const gchar * const filename,
	   const GufProtocol   protocol,
	   UserFilter *        filter )
{
  /*
   *  FIXME:  Check whether we have all required entries, fill them
   *  with fall-back values if necessary.
   *
   *  For now we live with a possible runtime bug (should be catched by
   *  other code anyway)
   */
  return guf_read (filename, protocol, filter);
}
