/*
 * (C) 2008  Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: guf-save.c,v 1.6 2008/06/11 17:52:03 tneuer Exp $
 *
 *  $Log: guf-save.c,v $
 *  Revision 1.6  2008/06/11 17:52:03  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.5  2008/06/07 16:35:58  tneuer
 *  Using g_strerror () instead of strerror ()
 *
 *  Revision 1.4  2008/06/05 15:23:52  tneuer
 *  Use g_fopen () for UTF-8 compatibility
 *
 *  Revision 1.3  2008/05/26 15:10:54  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.2  2008/05/21 12:30:32  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:05  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/22 15:25:44  tneuer
 *  Initial revision
 *
 */

#include "plugin-conf.h"

#include <string.h>
#include <errno.h>

#include <glib/gstdio.h>

#include "8bf.h"
#include "guf.h"


gboolean
guf_write ( const gchar * const filename,
	    const UserFilter *  filter )
{
  FILE * f;

  if ( (f = g_fopen (filename, "w")) != NULL )
    {
      GError * err = NULL;
      gchar  * buf;

      buf = g_key_file_to_data ((GKeyFile *)filter, NULL, &err);
      if (err != NULL)
	{
	  g_warning ("GUF %s '%s': %s\n",
		     _("Error saving"), filename, err->message );
	  g_error_free (err);
	  g_free (buf);
	  return FALSE;
	}
      fputs (buf, f);
      g_free (buf);
      fclose (f);
      return TRUE;
    }
  g_warning ("GUF %s '%s': %s\n",
	     _("Error saving"), filename, g_strerror(errno));
  return FALSE;
}


gboolean
guf_save ( const gchar * const filename,
	   const gchar * const comment,
	   const GufProtocol   protocol,
	   const UserFilter *  filter )
{
  if (protocol <= GUF_PROTOCOL) 
    {
      GError * err = NULL;

      if ( (comment != NULL) && (strlen (comment) > 0) )
	{
	  g_key_file_set_comment ((GKeyFile *)filter, NULL, NULL, comment, &err);
	  if (err != NULL)
	    {
	      g_warning ("GUF %s '%s': %s\n",
			 _("Error saving"), filename, err->message );
	      g_error_free (err);
	      return FALSE;
	    }
	}
      return guf_write (filename, filter);
    }
  g_warning ("GUF %s '%s': %s\n",
	     _("Error saving"), filename, _("Invalid protocol") );
  return FALSE;
}
