/*
 * (C) 1997,2003  Jens Ch. Restemeier <jrestemeier@currantbun.com>
 *
 * (C) 2008       Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: interface.c,v 1.43 2008/06/19 16:03:39 tneuer Exp $
 *
 *  $Log: interface.c,v $
 *  Revision 1.43  2008/06/19 16:03:39  tneuer
 *  Added popup-menus to notebook widget
 *
 *  Revision 1.42  2008/06/19 13:54:28  tneuer
 *  Minor optimizations in ctl toggle callback
 *
 *  Revision 1.41  2008/06/19 13:04:22  tneuer
 *  Group gtk_wdget_show () with gtk_widget_show_all () where applicable
 *
 *  Revision 1.40  2008/06/19 11:28:18  tneuer
 *  Bugfix: Missing state update for edge-mode editor component
 *
 *  Revision 1.39  2008/06/18 09:18:41  tneuer
 *  Add reset button for edge mode in control panel.
 *
 *  Revision 1.38  2008/06/17 15:41:14  tneuer
 *  Minor code cleanups
 *
 *  Revision 1.37  2008/06/17 15:31:35  tneuer
 *  Made some more function parameters const
 *
 *  Revision 1.36  2008/06/17 14:43:38  tneuer
 *  Added editor component for default edge mode
 *
 *  Revision 1.35  2008/06/17 13:53:12  tneuer
 *  Bugfix: Correctly set/toggle map tooltips
 *
 *  Revision 1.34  2008/06/16 16:34:39  tneuer
 *  Added guf_filter_(get|set)_edge_mode () access functions
 *
 *  Revision 1.33  2008/06/16 16:01:25  tneuer
 *  Added code-only optimization to transpiler
 *
 *  Revision 1.32  2008/06/16 09:40:09  tneuer
 *  Added reduction of source function to channel source function for constant 'z' where 0 <= z <= 3
 *
 *  Revision 1.31  2008/06/11 17:52:08  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.30  2008/06/09 12:05:38  tneuer
 *  Reduced box widget nesting in dialog (dropped outer vbox)
 *  Last filter filename now dynamically stored
 *
 *  Revision 1.29  2008/06/07 17:02:24  tneuer
 *  Using g_ascii_strcasecmp () instead of strcasecmp ()
 *
 *  Revision 1.28  2008/06/06 13:50:40  tneuer
 *  Open Gimp hint box when changing stand-alone run-mode flag in filter manager panel for the first time
 *
 *  Revision 1.27  2008/06/06 13:03:46  tneuer
 *  No more help button when running filters in stand-alone mode
 *
 *  Revision 1.26  2008/06/06 11:20:31  tneuer
 *  Handle "Control %u::Step" stettings in filter control panel
 *
 *  Revision 1.25  2008/06/04 23:24:31  tneuer
 *  Bugfix: filter controls are reset to defaults on re-run of plug-in in editor mode
 *
 *  Revision 1.24  2008/06/03 15:17:33  tneuer
 *  Bugfix: wrong locale of labels / tooltips in the filter control panel when re-running the plug-in
 *  Bugfix: wrong state of the filter editor panel when re-running the plug-in
 *
 *  Revision 1.23  2008/06/03 14:17:48  tneuer
 *  Added tooltips for expanders and formula entries in filter editor panel
 *
 *  Revision 1.22  2008/05/28 14:37:19  tneuer
 *  Using own Makefile for man-pages directory
 *
 *  Revision 1.21  2008/05/28 10:01:19  tneuer
 *  Using GtkPaned to divide preview and controls
 *
 *  Revision 1.20  2008/05/27 16:36:53  tneuer
 *  Filechooser dialogs: Use different filters for native / non-native files
 *
 *  Revision 1.19  2008/05/27 16:09:56  tneuer
 *  Filechooser dialogs: don't display hidden files / directories
 *  Filechooser dialogs: show filter searchpaths as shortcuts
 *  Filechooser dialogs: only show (potential) filter files
 *
 *  Revision 1.18  2008/05/27 15:09:23  tneuer
 *  Bugfix: development interface overwrites previous control values on re-run
 *
 *  Revision 1.17  2008/05/26 15:10:55  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.16  2008/05/26 13:03:12  tneuer
 *  Only display random seed reset widget if filter uses rnd() function
 *
 *  Revision 1.15  2008/05/26 12:01:36  tneuer
 *  Bugfix: editor panel load function has case sensitive fileextension match
 *  Bugfix: last filter filename is not invalidated if load fails
 *
 *  Revision 1.14  2008/05/22 11:03:43  tneuer
 *  New callback functions for source and meta-info entry widgets.
 *
 *  Revision 1.13  2008/05/21 15:26:59  tneuer
 *  Default extension for saving files is now .guf
 *
 *  Revision 1.12  2008/05/21 12:30:36  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.11  2008/05/19 10:55:23  tneuer
 *  Code cleanup: merge duplicate code in plug-in dialog(s)
 *
 *  Revision 1.10  2008/05/16 16:37:15  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.9  2008/05/16 15:43:57  tneuer
 *  Added Gimp menu registry toggle to filter manager panel.
 *
 *  Revision 1.8  2008/05/16 14:02:52  tneuer
 *  Loading stand-alone (registered) filters.
 *
 *  Revision 1.7  2008/05/14 15:26:50  tneuer
 *  Bugfix: map toggle uses values incorrectly on load.
 *
 *  Revision 1.6  2008/05/14 14:29:59  tneuer
 *  Code cleanup prior to editor panel rewrite.
 *
 *  Revision 1.5  2008/05/13 15:03:38  tneuer
 *  Using gimp_config_ functions to build paths.
 *
 *  Revision 1.4  2008/05/13 11:03:06  tneuer
 *  GUF compile fixes.
 *
 *  Revision 1.3  2008/05/05 13:32:41  tneuer
 *  New functions guf_filter_renew_from_pdb () and guf_filter_dump () for PDB operations.
 *
 *  Revision 1.2  2008/04/27 15:25:21  tneuer
 *  *** empty log message ***
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:02  tneuer
 *  initial import
 */

#include "plugin-conf.h"

#include <libgen.h>
#include <stdlib.h>
#include <string.h>

#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>

#define __USERFILTER_INTERNAL__ TRUE

#include "main.h"
#include "fileio.h"
#include "scanner.h"
#include "render.h"
#include "checksum.h"
#include "interface.h"
#include "optimizer.h"
#include "interpreter.h"
#include "scan-filters.h"

#include "8bf.h"
#include "afs.h"
#include "txt.h"
#include "guf.h"


/*  Constants  */

enum
  {                     /* filter manager panel / filter list store */
    COL_REGISTER = 0,   /* is filter registered in the menu ? */
    COL_CATEGORY,       /* filter factory category / gimp menu path */
    COL_TITLE,          /* filter title / name */
    COL_AUTHOR,         /* filter author(s) */
    COL_COPYRIGHT,      /* copyright / license notice */
    COL_FNAME,          /* filter filename (.guf) */
    MGR_NCOLS
  };


/*  Local variables  */

enum {
  EXPAND_META = 0,
  EXPAND_CTRL,
  EXPAND_CODE,
  N_EXPANDERS
};

static GtkWidget * dialog_category    = NULL;      /* edit entries... */
static GtkWidget * dialog_title       = NULL;
static GtkWidget * dialog_author      = NULL;
static GtkWidget * dialog_copyright   = NULL;
static GtkWidget * dialog_description = NULL;
static GtkWidget * dialog_edge_edit   = NULL;
static GtkWidget * dialog_expander[N_EXPANDERS] = {
  NULL, NULL, NULL
};
static GtkWidget * dialog_source[N_CHANNELS]   = { /* filter source code */
  NULL, NULL, NULL, NULL
};
static GtkWidget * dialog_menable[N_MAPS] = {      /* editor: map() widgets... */
  NULL, NULL, NULL, NULL
};
static GtkWidget * dialog_mlabel[N_MAPS] = {
  NULL, NULL, NULL, NULL
};
static GtkWidget * dialog_slabel[N_CONTROLS] = {   /* editor: ctl() widgets... */
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
};
static GtkObject * dialog_svalue[N_CONTROLS] = {
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
};
static GtkWidget * dialog_spnbtn[N_CONTROLS] = {
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
};
static GtkWidget * dialog_senable[N_CONTROLS] = {
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
};
static gchar * last_filter = NULL;                 /* last filter used */


/*  Callback Function Prototypes */

static void cb_editor_map_toggle (const GtkWidget * widget, const gpointer data);
static void cb_editor_ctl_toggle (const GtkWidget * widget, const gpointer data);


/*  Local functions  */

static void
update_dialog_title (void)
{
  gchar * buf;
  gchar * ttl;

  ttl = guf_filter_get_locale_title (filter);
  buf = g_strdup_printf ("%s: %s", _("User Filter"), ttl);
  gtk_window_set_title (GTK_WINDOW (dialog_dialog), buf);
  g_free (buf);
  g_free (ttl);
}


static gchar *
is_bin_filter (const gchar * const s)
{
  gchar * x;

  if (((x = strrchr (s, '.')) != NULL) &&
      !g_ascii_strcasecmp (x, EXT_8BF))
    {
      return x;
    }
  return NULL;
}


static void
uf_slider_show (const guint slider)
{
  GtkObject * scale_entry = dialog_slider[slider];

  gtk_widget_show (GIMP_SCALE_ENTRY_LABEL (scale_entry));
  gtk_widget_show (GIMP_SCALE_ENTRY_SCALE (scale_entry));
  gtk_widget_show (GIMP_SCALE_ENTRY_SPINBUTTON (scale_entry));
  gtk_widget_show (dialog_sreset[slider]);
}


static inline void
uf_entry_set_guf_text (GtkEntry * entry,
		       gchar *    value)
{
  gtk_entry_set_text (entry, value);
  g_free (value);
}


static void
uf_update_internals (const gboolean defaults)
{
  guint n;

  /*
   *  FIXME:  This function needs to be cleaned up.
   *          It is a dirty mess (yet, it seems to work)...
   */
  update_dialog_title ();

  for (n = 0; n < N_CONTROLS; n++)
    {
      gboolean enabled;
      gchar *  label;
      gchar *  tooltip;
      gchar *  entry;
      gint     preset;
      gint     step;

      enabled = guf_filter_get_ctl_enabled (filter, n);
      label   = guf_filter_get_locale_ctl_label (filter, n);
      tooltip = guf_filter_get_locale_ctl_tooltip (filter, n);
      entry   = guf_filter_get_ctl_label (filter, n);
      preset  = guf_filter_get_ctl_preset (filter, n);
      step    = guf_filter_get_ctl_step (filter, n);

      if (defaults)
	{
	  control.value[n] = preset;
	  gtk_adjustment_set_value (GIMP_SCALE_ENTRY_SCALE_ADJ (dialog_slider[n]), preset);
	  g_object_set (GIMP_SCALE_ENTRY_SCALE_ADJ (dialog_slider[n]),
			"lower", (gdouble)0,
			"upper", (gdouble)(255 - (255 % step)),
			"step-increment", (gdouble)step,
			NULL);
	  gtk_adjustment_set_value (GIMP_SCALE_ENTRY_SPINBUTTON_ADJ (dialog_slider[n]), preset);
	  g_object_set (GIMP_SCALE_ENTRY_SPINBUTTON_ADJ (dialog_slider[n]),
			"lower", (gdouble)0,
			"upper", (gdouble)(255 - (255 % step)),
			"step-increment", (gdouble)step,
			NULL);
	}
      gtk_label_set_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[n]), label);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[n]), tooltip);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SCALE (dialog_slider[n]), tooltip);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SPINBUTTON (dialog_slider[n]), tooltip);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog_senable[n]), enabled);
      cb_editor_ctl_toggle (dialog_senable[n], GINT_TO_POINTER (n));

      gtk_entry_set_text (GTK_ENTRY (dialog_slabel[n]), entry);
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog_spnbtn[n]), preset);

      if (!enabled)
	{
	  _uf_slider_hide (n);
	}
      else
	{
	  uf_slider_show (n);
	}
      g_free (tooltip);
      g_free (label);
    }

  uf_entry_set_guf_text (GTK_ENTRY (dialog_category), guf_filter_get_category (filter));
  uf_entry_set_guf_text (GTK_ENTRY (dialog_title), guf_filter_get_title (filter));
  uf_entry_set_guf_text (GTK_ENTRY (dialog_author), guf_filter_get_author (filter));
  uf_entry_set_guf_text (GTK_ENTRY (dialog_copyright), guf_filter_get_copyright (filter));
  uf_entry_set_guf_text (GTK_ENTRY (dialog_description), guf_filter_get_description (filter));

  if (defaults)
    {
      gtk_combo_box_set_active (GTK_COMBO_BOX (dialog_edge),
				control.edge_mode = guf_filter_get_edge_mode (filter));

    }
  gtk_combo_box_set_active (GTK_COMBO_BOX (dialog_edge_edit),
			    guf_filter_get_edge_mode (filter));

  for (n = 0; n < N_MAPS; n++)
    {
      gboolean enabled = guf_filter_get_map_enabled (filter, n);
      gchar *  label   = guf_filter_get_locale_map_label (filter, n);
      gchar *  entry   = guf_filter_get_map_label (filter, n);
      gchar *  tooltip = guf_filter_get_locale_map_tooltip (filter, n);

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog_menable[n]), enabled);
      cb_editor_map_toggle (dialog_menable[n], GINT_TO_POINTER (n));
      gtk_entry_set_text (GTK_ENTRY (dialog_mlabel[n]), entry);

      if (enabled)
	{
	  const guint ctl = 2 * n;

	  gtk_label_set_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl]), label);
	  gtk_label_set_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl + 1]), "");
	  gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl]), tooltip);
	  gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl + 1]), tooltip);
	  gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SCALE (dialog_slider[ctl]), tooltip);
	  gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SCALE (dialog_slider[ctl + 1]), tooltip);
	  gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SPINBUTTON (dialog_slider[ctl]), tooltip);
	  gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SPINBUTTON (dialog_slider[ctl + 1]), tooltip);
	  uf_slider_show (ctl);
	  uf_slider_show (ctl + 1);
	}
      g_free (label);
      g_free (entry);
      g_free (tooltip);
    }

  for (n = 0; n < N_CHANNELS; n++)
    {
      static const gchar * const rgba [N_CHANNELS] = {
	"r", "g", "b", "a"
      };
      gchar * src;

      src = guf_filter_get_src (filter, n);
      gtk_entry_set_text (GTK_ENTRY (dialog_source[n]), src);

      if (uf_code[n])
	{
	  free_code_tree (uf_code[n]);
	}
      if (!(uf_code[n] = get_code_tree (src)))
	{
	  /*
	   *  FIXME: report channel name (and maybe dump the code)
	   */
	  g_warning (_("Syntax error in code\n"));

	  /*
	   *  Set to default value to avoid bugging the preview...
	   */
	  uf_code[n] = get_code_tree (rgba[n]);
	}
      else
	{
	  optimize_code_tree (uf_code[n], n, FALSE, FALSE, &environment);
	}
      g_free (src);
    }

  if (_filter_uses_rnd (filter))
    {
      gtk_widget_show (dialog_rnd);
    }
  else
    {
      gtk_widget_hide (dialog_rnd);
    }
}


/*********************************************************************
 *
 *  Filter Control Panel
 */

/*  shared code with userfilter.c  */


/*********************************************************************
 *
 *  Filter Editor Panel
 */

static void
cb_editor_map_update (const GtkWidget * widget,
		      const gpointer    data)
{
  const guint   map = GPOINTER_TO_INT (data);
  const guint   ctl = 2 * map;
  const gchar * lbl = gtk_entry_get_text (GTK_ENTRY (widget));

  guf_filter_set_map_label (filter, map, lbl);
  gtk_label_set_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl]), lbl);
}


static void
cb_editor_ctl_update (const GtkWidget * widget,
		      const gpointer    data)
{
  const guint   ctl = GPOINTER_TO_INT (data);
  const gchar * val = gtk_entry_get_text (GTK_ENTRY (widget));

  guf_filter_set_ctl_label (filter, ctl, val);
  gtk_label_set_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl]), val);
}


typedef gchar * (* UpdateFunc) (UserFilter *  filter,
				const gchar * data);

static void
cb_editor_info_update (const GtkWidget * widget,
		       const UpdateFunc  update)
{
  update (filter, gtk_entry_get_text (GTK_ENTRY (widget)));
  if (widget == dialog_title)
    {
      update_dialog_title ();
    }
}


static void
cb_editor_source_update (const GtkWidget * widget,
			 const gpointer    data)
{
  const guint chan = GPOINTER_TO_INT (data);

  guf_filter_set_src (filter, chan,
		      gtk_entry_get_text (GTK_ENTRY (widget)));
  /*
   *  FIXME:  This would be a nice point to set up an on-the-fly
   *          error check of the code entered.
   *          We need to disable the display of a message box in
   *          case of a syntax error first - then we need to have
   *          an icon that can be toggled in order to display the
   *          fitness of the code (ok, error).
   */

  if (_filter_uses_rnd (filter))
    {
      gtk_widget_show (dialog_rnd);
    }
  else
    {
      gtk_widget_hide (dialog_rnd);
    }
}


static void
cb_editor_spnbtn_update (const GtkWidget * widget,
			 const gpointer    data)
{
  const guint ctl = GPOINTER_TO_INT (data);
  const gint  val = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (widget));

  guf_filter_set_ctl_preset (filter, ctl, val);
}


static void
cb_editor_compile (const GtkWidget * widget,
		   const gpointer    data)
{
  const guint k = GPOINTER_TO_INT (data);
  CodeTree *  code;
  gchar *     src;

  src = guf_filter_get_src (filter, k);
  if ((code = get_code_tree (src)))
    {
      /*
       *  Keep old code tree if syntax error occurs
       */
      free_code_tree (uf_code[k]);
      optimize_code_tree (code, k, FALSE, FALSE, &environment);
      uf_code[k] = code;
    }
  g_free (src);
}


static void
cb_editor_filter_new (const GtkWidget * widget,
		      const gpointer    data)
{
  filter = guf_filter_renew (filter);
  uf_update_internals (TRUE);
}


static void
uf_editor_add_chooser_filetype (GtkFileFilter *     filter,
				const gchar * const ext)
{
  gchar * pattern;

  pattern = g_strdup_printf ("%c%s", '*', ext);
  gtk_file_filter_add_pattern (filter, pattern);
  g_free (pattern);
}


static void
uf_editor_add_chooser_shortcut_folders (GtkFileChooser *    chooser,
					const gchar * const path)
{
  if (path != NULL &&
      strlen(path) > 0)
    {
      gchar ** p;
      gint     i;

      p = g_strsplit (path, G_SEARCHPATH_SEPARATOR_S, 0);
      for (i=0; p[i] != NULL; i++)
	{
	  GError * err = NULL;

	  if (!gtk_file_chooser_add_shortcut_folder (chooser, p[i], &err))
	    {
	      g_warning ("%s '%s': %s", _("Failed to add shortcut folder"), p[i], err->message);
	      g_error_free (err);
	    }
	}
      g_strfreev (p);
    }
}


static void
cb_editor_filter_load (const GtkWidget * widget,
		       const gpointer    data)
{
  GtkFileFilter * pattern;
  GtkWidget *     chooser;

  chooser = gtk_file_chooser_dialog_new (_("Load Filter"),
					 GTK_WINDOW (data),
					 GTK_FILE_CHOOSER_ACTION_OPEN,
					 GTK_STOCK_CANCEL,
					 GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN,
					 GTK_RESPONSE_ACCEPT, NULL);

  pattern = gtk_file_filter_new ();
  uf_editor_add_chooser_filetype (pattern, EXT_GUF);
  gtk_file_filter_set_name (pattern, _("User Filter files (*.guf)"));
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), pattern);

  pattern = gtk_file_filter_new ();
  uf_editor_add_chooser_filetype (pattern, EXT_8BF);
  uf_editor_add_chooser_filetype (pattern, EXT_AFS);
  uf_editor_add_chooser_filetype (pattern, EXT_TXT);
  gtk_file_filter_set_name (pattern, _("Filter Factory files (*.8bf,*.afs,*.txt)"));
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), pattern);

  pattern = gtk_file_filter_new ();
  uf_editor_add_chooser_filetype (pattern, ".*");
  gtk_file_filter_set_name (pattern, _("All files (*.*)"));
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), pattern);

  uf_editor_add_chooser_shortcut_folders (GTK_FILE_CHOOSER (chooser), filter_path);

  gtk_file_chooser_set_show_hidden (GTK_FILE_CHOOSER (chooser), FALSE);

  if (last_filter != NULL)
    {
      gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (chooser), last_filter);
    }

  if (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_ACCEPT)
    {
      gchar * filename;

      guf_filter_free (filter);
      filter = guf_filter_new ();
      filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));

      if (!load_filter (filename, filter))
	{
	  g_warning (_("Error loading filter '%s'\n"), filename);
	  filter = guf_filter_renew (filter);
	  g_free (filename);
	  g_free (last_filter);
	  last_filter = NULL;
	}
      else
	{
	  gchar * x;

	  if ((x = is_bin_filter(filename)) != NULL)
	    {
	      strcpy (x, EXT_GUF);
	    }
	  g_free (last_filter);
	  last_filter = filename;
	}
      uf_update_internals (TRUE);
    }

  gtk_widget_destroy (chooser);
}


static void
cb_editor_map_toggle (const GtkWidget * widget,
		      const gpointer    data)
{
  const guint    map      = GPOINTER_TO_INT (data);
  const guint    ctl      = 2 * map;
  const gboolean map_on   = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
  const gboolean ctl_0_on = guf_filter_get_ctl_enabled (filter, ctl);
  const gboolean ctl_1_on = guf_filter_get_ctl_enabled (filter, ctl + 1);

  guf_filter_set_map_enabled (filter, map, map_on);

  /*
   *  Active map means no label (and no disabling of value) for both controllers.
   */

  gtk_widget_set_sensitive (dialog_mlabel[map], map_on);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog_senable[ctl]), map_on || ctl_0_on);

  gtk_widget_set_sensitive (dialog_senable[ctl], !map_on);
  gtk_widget_set_sensitive (dialog_slabel[ctl],  !map_on && ctl_0_on);
  gtk_widget_set_sensitive (dialog_spnbtn[ctl],   map_on || ctl_0_on);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog_senable[ctl + 1]), map_on || ctl_1_on);

  gtk_widget_set_sensitive (dialog_senable[ctl + 1], !map_on);
  gtk_widget_set_sensitive (dialog_slabel[ctl + 1],  !map_on && ctl_1_on);
  gtk_widget_set_sensitive (dialog_spnbtn[ctl + 1],   map_on || ctl_1_on);

  /*
   *  Now switch the labels of the control panel, too...
   */

  if (map_on)
    {
      gchar * label   = guf_filter_get_locale_map_label (filter, map);
      gchar * tooltip = guf_filter_get_locale_map_tooltip (filter, map);

      gtk_label_set_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl]), label);
      gtk_label_set_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl + 1]), "");
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl]), tooltip);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl + 1]), tooltip);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SCALE (dialog_slider[ctl]), tooltip);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SCALE (dialog_slider[ctl + 1]), tooltip);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SPINBUTTON (dialog_slider[ctl]), tooltip);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SPINBUTTON (dialog_slider[ctl + 1]), tooltip);
      g_free (tooltip);
      g_free (label);
    }
  else
    {
      gchar * ctl0 = guf_filter_get_locale_ctl_label (filter, ctl);
      gchar * ctl1 = guf_filter_get_locale_ctl_label (filter, ctl + 1);
      gchar * tip0 = guf_filter_get_locale_ctl_tooltip (filter, ctl);
      gchar * tip1 = guf_filter_get_locale_ctl_tooltip (filter, ctl + 1);

      gtk_label_set_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl]), ctl0);
      gtk_label_set_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl + 1]), ctl1);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl]), tip0);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl + 1]), tip1);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SCALE (dialog_slider[ctl]), tip0);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SCALE (dialog_slider[ctl + 1]), tip1);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SPINBUTTON (dialog_slider[ctl]), tip0);
      gtk_widget_set_tooltip_text (GIMP_SCALE_ENTRY_SPINBUTTON (dialog_slider[ctl + 1]), tip1);
      g_free (ctl1);
      g_free (ctl0);
      g_free (tip1);
      g_free (tip0);
    }
}


static void
cb_editor_ctl_toggle (const GtkWidget * widget,
		      const gpointer    data)
{
  const gboolean enabled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
  const guint    ctl     = GPOINTER_TO_INT (data);
  const guint    map     = ctl / 2;
  const guint    mod     = 2 * map;

  guf_filter_set_ctl_enabled (filter, ctl, enabled);

  /*
   *  Toggle sensitivity of controller entries (label and value)
   */
  gtk_widget_set_sensitive (dialog_slabel[ctl], enabled);
  gtk_widget_set_sensitive (dialog_spnbtn[ctl], enabled);

  /*
   *  If both controllers are active, also enable the map toggle
   */
  gtk_widget_set_sensitive (dialog_menable[map],
			    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog_senable[mod]))
			    &&
			    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog_senable[mod + 1])));

  /*
   *  Now switch the label of the control panel, too...
   */

  if (enabled)
    {
      gchar * lbl;

      lbl = guf_filter_get_locale_ctl_label (filter, ctl);
      gtk_label_set_text (GIMP_SCALE_ENTRY_LABEL (dialog_slider[ctl]), lbl);
      uf_slider_show (ctl);
      g_free (lbl);
    }
  else
    {
      _uf_slider_hide (ctl);
    }
}


static void
cb_editor_filter_save_as (const GtkWidget * widget,
			  const gpointer    data)
{
  GtkFileFilter * pattern;
  GtkWidget *     chooser;

  chooser = gtk_file_chooser_dialog_new (_("Save Filter"),
					 GTK_WINDOW (data),
					 GTK_FILE_CHOOSER_ACTION_SAVE,
					 GTK_STOCK_CANCEL,
					 GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE,
					 GTK_RESPONSE_ACCEPT, NULL);

  gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (chooser), TRUE);

  pattern =  gtk_file_filter_new ();
  uf_editor_add_chooser_filetype (pattern, EXT_GUF);
  gtk_file_filter_set_name (pattern, _("User Filter files (*.guf)"));
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), pattern);

  pattern =  gtk_file_filter_new ();
  uf_editor_add_chooser_filetype (pattern, EXT_AFS);
  uf_editor_add_chooser_filetype (pattern, EXT_TXT);
  gtk_file_filter_set_name (pattern, _("Filter Factory files (*.afs,*.txt)"));
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), pattern);

  pattern =  gtk_file_filter_new ();
  uf_editor_add_chooser_filetype (pattern, ".*");
  gtk_file_filter_set_name (pattern, _("All files (*.*)"));
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), pattern);

  uf_editor_add_chooser_shortcut_folders (GTK_FILE_CHOOSER (chooser), filter_path);

  gtk_file_chooser_set_show_hidden (GTK_FILE_CHOOSER (chooser), FALSE);

  if (last_filter == NULL)
    {
      gchar * fn;

      fn = g_strdup_printf ("%s%s", _("untitled"), EXT_GUF);
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), g_get_home_dir ());
      gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (chooser), fn);
      g_free (fn);
    }
  else
    {
      gchar * x;
      gchar * p = g_path_get_basename (last_filter);

      if ((x = is_bin_filter (p)))
	{
	  strcpy (x, EXT_GUF);
	}
      gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (chooser), p);
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
					   ((x = g_path_get_dirname (last_filter)) &&
					    (x[0] != '.')) ? x : g_get_home_dir ());
      g_free (x);
      g_free (p);
    }

  if (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_ACCEPT)
    {
      g_free (last_filter);
      last_filter = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
      save_filter (last_filter, filter);
    }

  gtk_widget_destroy (chooser);
}


static void
cb_editor_filter_save (const GtkWidget * widget,
		       const gpointer    data)
{
  if ((last_filter == NULL) ||
      is_bin_filter (last_filter))
    {
      /*
       *  Either no filename has been set for the filter,
       *  or is invalid (empty),
       *  or is set from the name of a Windows binary filter (.8bf),
       *  or is set from an entry in a filter library (.ffl).
       */
      cb_editor_filter_save_as (widget, data);
    }
  else
    {
      save_filter (last_filter, filter);
    }
}


static void
cb_editor_edge_mode_update (const GtkWidget * widget,
			    const gpointer    data)
{
  guf_filter_set_edge_mode (filter, gtk_combo_box_get_active (GTK_COMBO_BOX (widget)));
}


/*  Local functions  */

static GtkWidget *
uf_editor_meta_entry (GtkWidget *         table,
		      const guint         line,
		      const guint         col,
		      const gchar * const text,
		      const gchar * const tooltip,
		      gchar *             data,
		      const UpdateFunc    update)
{
  GtkWidget * widget;
  const guint c = 2 * col;

  widget = gtk_label_new (text);
  gtk_label_set_justify (GTK_LABEL (widget), GTK_JUSTIFY_RIGHT);
  gtk_widget_set_tooltip_text (widget, tooltip);
  gtk_table_attach (GTK_TABLE (table), widget, c, c + 1, line, line + 1,
		    GTK_SHRINK, GTK_SHRINK, 5, 2);

  widget = gtk_entry_new ();
  gtk_entry_set_max_length (GTK_ENTRY (widget), F_LBL_LEN);
  gtk_widget_set_tooltip_text (widget, tooltip);
  g_signal_connect (GTK_OBJECT (widget), "changed",
		    G_CALLBACK (cb_editor_info_update), update);
  gtk_table_attach (GTK_TABLE (table), widget, c + 1, c + 2, line, line + 1,
		    GTK_FILL | GTK_EXPAND, GTK_SHRINK, 5, 2);
  gtk_entry_set_text (GTK_ENTRY (widget), data);

  /*
   *  "data" has previously been allocated, we need to free it again...
   */
  g_free (data);

  return widget;
}


static inline GtkWidget *
uf_editor_fi_map_entry (GtkTable *          table,
			const guint         row,
			const guint         col,
			const gchar * const tooltip,
			const gchar * const data)
{
  GtkWidget * widget;
  gchar *     buf;

  buf = g_strdup_printf ("map(%d):", row);

  widget = gtk_label_new (buf);
  gtk_table_attach (table, widget, col, col + 1, row, row + 1, GTK_SHRINK,
		    GTK_SHRINK, 2, 2);
  gtk_widget_set_tooltip_text (widget, tooltip);

  widget = gtk_check_button_new ();
  gtk_table_attach (table, widget, col + 1, col + 2, row, row + 1, GTK_SHRINK,
		    GTK_SHRINK, 0, 2);
  gtk_widget_set_tooltip_text (widget,
			       _("Enable or disable mapped control on this contoller group..."));
  g_signal_connect (GTK_OBJECT (widget), "toggled",
		    G_CALLBACK (cb_editor_map_toggle), GINT_TO_POINTER (row));
  dialog_menable[row] = widget;

  widget = gtk_entry_new ();
  gtk_entry_set_max_length (GTK_ENTRY (widget), F_LBL_LEN);
  g_signal_connect (GTK_OBJECT (widget), "changed",
		    G_CALLBACK (cb_editor_map_update), GINT_TO_POINTER (row));
  gtk_table_attach (table, widget, col + 2, col + 3, row, row + 1,
		    GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 2);
  gtk_widget_set_tooltip_text (widget, tooltip);
  gtk_entry_set_text (GTK_ENTRY (widget), data);

  g_free (buf);

  return widget;
}


static GtkWidget *
uf_editor_fi_ctl_entry (GtkTable *          table,
			const guint         row,
			const guint         col,
			const guint         ctl,
			const gchar * const tooltip,
			const gchar * const data)
{
  GtkWidget * widget;
  gchar *     buf;

  buf = g_strdup_printf ("ctl(%d):", ctl);

  widget = gtk_label_new (buf);
  gtk_table_attach (GTK_TABLE (table), widget, col, col + 1, row, row + 1,
		    GTK_SHRINK, GTK_SHRINK, 2, 2);
  gtk_widget_set_tooltip_text (widget, tooltip);

  widget = gtk_check_button_new ();
  gtk_table_attach (table, widget, col + 1, col + 2, row, row + 1, GTK_SHRINK,
		    GTK_SHRINK, 0, 2);
  gtk_widget_set_tooltip_text (widget,
			       _("Enable or disable this contoller..."));
  g_signal_connect (GTK_OBJECT (widget), "toggled",
		    G_CALLBACK (cb_editor_ctl_toggle), GINT_TO_POINTER (ctl));
  dialog_senable[ctl] = widget;

  widget = gtk_entry_new ();
  gtk_entry_set_max_length (GTK_ENTRY (widget), F_LBL_LEN);
  g_signal_connect (GTK_OBJECT (widget), "changed",
		    G_CALLBACK (cb_editor_ctl_update), GINT_TO_POINTER (ctl));
  gtk_table_attach (GTK_TABLE (table), widget, col + 2, col + 3, row, row + 1,
		    GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 2);
  gtk_widget_set_tooltip_text (widget, tooltip);
  gtk_entry_set_text (GTK_ENTRY (widget), data);

  g_free (buf);

  return widget;
}


static GtkObject *
uf_editor_fi_ctl_spinbutton (GtkTable *          table,
			     const guint         row,
			     const guint         col,
			     const guint         ctl,
			     const gchar * const tooltip)
{
  GtkWidget * widget;
  GtkObject * adj;

  widget = gimp_spin_button_new (&adj, 0, 0, 255, /* value, lower, upper */
				 1, 1,	          /* step increment, page increment */
				 25, 1, 0);	  /* page size, climb rate, digits */

  g_signal_connect (GTK_OBJECT (widget), "value-changed",
		    G_CALLBACK (cb_editor_spnbtn_update),
		    GINT_TO_POINTER (ctl));

  gtk_table_attach (GTK_TABLE (table), widget, col, col + 1, row, row + 1,
		    GTK_FILL | GTK_EXPAND, GTK_SHRINK, 5, 2);
  gtk_widget_set_tooltip_text (widget, tooltip);

  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget),
			     guf_filter_get_ctl_preset (filter, ctl));

  dialog_spnbtn[ctl] = widget;

  return adj;
}


static inline void
uf_editor_filter_interface (GtkTable *   table,
			    const guint  row,
			    GtkWidget ** maplabel,
			    GtkWidget ** ctllabel0,
			    GtkObject ** ctlvalue0,
			    GtkWidget ** ctllabel1,
			    GtkObject ** ctlvalue1)
{
  const guint ctl = 2 * row;
  gchar *     lbl;

  lbl = guf_filter_get_map_label (filter, row);
  *maplabel = uf_editor_fi_map_entry (table, row, 0,
				      _("Label for the mapped controller pair..."), lbl);
  g_free (lbl);

  lbl = guf_filter_get_ctl_label (filter, ctl);
  *ctllabel0 = uf_editor_fi_ctl_entry (table, row, 3, ctl,
				       _("Label for this controller..."), lbl);
  g_free (lbl);

  *ctlvalue0 = uf_editor_fi_ctl_spinbutton (table, row, 6, ctl,
					    _("Initial value for this controller..."));

  lbl = guf_filter_get_ctl_label (filter, ctl + 1);
  *ctllabel1 = uf_editor_fi_ctl_entry (table, row, 7, ctl + 1,
				       _("Label for this controller..."), lbl);
  g_free (lbl);

  *ctlvalue1 = uf_editor_fi_ctl_spinbutton (table, row, 10, ctl + 1,
					    _("Initial value for this controller..."));
}


static inline void
uf_build_page_filter_editor (GtkWidget * notebook)
{
  static const gchar * const channel[N_CHANNELS] = {
    GIMP_STOCK_CHANNEL_RED,
    GIMP_STOCK_CHANNEL_GREEN,
    GIMP_STOCK_CHANNEL_BLUE,
    GIMP_STOCK_CHANNEL
  };
  GtkWidget * hbox;
  GtkWidget * vbox;
  GtkWidget * vbox2;
  GtkWidget * table;
  GtkWidget * label;
  GtkWidget * button;
  guint       i;

  vbox = _uf_notebook_tab (notebook,
			   GIMP_STOCK_EDIT,
			   _("Filter Editor"),
			   _("Create or modify a filter..."));

  /*
   *  Meta information editor
   */
  dialog_expander[EXPAND_META] = gtk_expander_new (_("Filter Information"));
  gtk_box_pack_start (GTK_BOX (vbox), dialog_expander[EXPAND_META], FALSE, FALSE, 0);
  gtk_expander_set_expanded (GTK_EXPANDER(dialog_expander[EXPAND_META]), TRUE);
  gtk_widget_set_tooltip_text (dialog_expander[EXPAND_META],
			       _("General information about the filter..."));

  table = gtk_table_new (3, 4, FALSE);
  gtk_container_add (GTK_CONTAINER (dialog_expander[EXPAND_META]), table);

  /*
   *  FIXME:  We need a date / version information entry field.
   */
  dialog_category =
    uf_editor_meta_entry (table, 0, 0, _("Category:"),
			  _("Enter the filter category here..."),
			  guf_filter_get_category (filter),
			  guf_filter_set_category);
  dialog_title =
    uf_editor_meta_entry (table, 0, 1, _("Title:"),
			  _("Enter the name of the filter here..."),
			  guf_filter_get_title (filter),
			  guf_filter_set_title);
  dialog_author =
    uf_editor_meta_entry (table, 1, 0, _("Author:"),
			  _("Enter the name of the filter's author here..."),
			  guf_filter_get_author (filter),
			  guf_filter_set_author);
  dialog_copyright =
    uf_editor_meta_entry (table, 1, 1, _("Copyright:"),
			  _("Enter the copyright information for the filter here..."),
			  guf_filter_get_copyright (filter),
			  guf_filter_set_copyright);
  dialog_description =
    uf_editor_meta_entry (table, 2, 0, _("Description:"),
			  _("Enter a description for the filter here..."),
			  guf_filter_get_description (filter),
			  guf_filter_set_description);

  /*
   *  Interface editor
   *
   *  FIXME: The controls part of the editor is way too big and we still need
   *  to add the tooltip entries...
   *  We'll probably need another expander for each control pair.
   */
  dialog_expander[EXPAND_CTRL] = gtk_expander_new (_("Filter Interface"));
  gtk_box_pack_start (GTK_BOX (vbox), dialog_expander[EXPAND_CTRL], FALSE, FALSE, 0);
  gtk_expander_set_expanded (GTK_EXPANDER (dialog_expander[EXPAND_CTRL]), FALSE);
  gtk_widget_set_tooltip_text (dialog_expander[EXPAND_CTRL],
			       _("Interface settings for the filter..."));

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (dialog_expander[EXPAND_CTRL]), vbox2);

  table = gtk_table_new (4, 11, FALSE);
  gtk_box_pack_start (GTK_BOX (vbox2), table, FALSE, FALSE, 0);
  for (i = 0; i < N_MAPS; i++)
    {
      uf_editor_filter_interface (GTK_TABLE (table), i,
				  &dialog_mlabel[i],
				  &dialog_slabel[2 * i],
				  &dialog_svalue[2 * i],
				  &dialog_slabel[1 + 2 * i],
				  &dialog_svalue[1 + 2 * i]);
    }


  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);

  label = gtk_label_new (_("Default edge mode:"));
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  dialog_edge_edit = gtk_combo_box_new_text ();
  gtk_box_pack_start (GTK_BOX (hbox), dialog_edge_edit, FALSE, FALSE, 0);
  gtk_combo_box_append_text (GTK_COMBO_BOX (dialog_edge_edit), _("None"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (dialog_edge_edit), _("Wrap"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (dialog_edge_edit), _("Smear"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (dialog_edge_edit), _("Black"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (dialog_edge_edit), _("Background"));
  gtk_combo_box_set_active (GTK_COMBO_BOX (dialog_edge_edit), guf_filter_get_edge_mode (filter));
  g_signal_connect (GTK_OBJECT (dialog_edge_edit), "changed",
		    G_CALLBACK (cb_editor_edge_mode_update), NULL);
  gtk_widget_set_tooltip_text (hbox,
			       _("Change the default behaviour of the image source functions for off-image pixels..."));

  /*
   *  Formula editor
   */
  dialog_expander[EXPAND_CODE] = gtk_expander_new (_("Filter Formulas"));
  gtk_box_pack_start (GTK_BOX (vbox), dialog_expander[EXPAND_CODE], FALSE, FALSE, 0);
  gtk_expander_set_expanded (GTK_EXPANDER (dialog_expander[EXPAND_CODE]), FALSE);
  gtk_widget_set_tooltip_text (dialog_expander[EXPAND_CODE],
			       _("Formulas used in the filter..."));

  table = gtk_table_new (4, 3, FALSE);
  gtk_container_add (GTK_CONTAINER (dialog_expander[EXPAND_CODE]), table);

  for (i = 0; i < N_CHANNELS; i++)
    {
      static const gchar * const RGBA[N_CHANNELS] = {
	N_("Red channel formula ('r' or 'c' for leaving it unchanged)..."),
	N_("Green channel formula ('g' or 'c' for leaving it unchanged)..."),
	N_("Blue channel formula ('b' or 'c' for leaving it unchanged)..."),
	N_("Alpha channel formula ('a' or 'c' for leaving it unchanged)..."),
      };
      gchar * src;

      label = gtk_image_new_from_stock (channel[i], GTK_ICON_SIZE_MENU);
      gtk_table_attach (GTK_TABLE (table), label, 0, 1, i, i + 1, GTK_SHRINK,
			GTK_SHRINK, 5, 2);
      gtk_widget_set_tooltip_text (label, gettext (RGBA[i]));

      dialog_source[i] = gtk_entry_new ();
      gtk_entry_set_max_length (GTK_ENTRY (dialog_source[i]), F_SRC_LEN);
      g_signal_connect (GTK_OBJECT (dialog_source[i]), "changed",
			G_CALLBACK (cb_editor_source_update),
			GINT_TO_POINTER (i));
      gtk_table_attach (GTK_TABLE (table), dialog_source[i], 1, 2, i, i + 1,
			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 5, 2);
      gtk_widget_set_tooltip_text (dialog_source[i], gettext (RGBA[i]));
      src = guf_filter_get_src (filter, i);
      gtk_entry_set_text (GTK_ENTRY (dialog_source[i]), src);
      g_free (src);

      button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
      gtk_table_attach (GTK_TABLE (table), button, 2, 3, i, i + 1, GTK_SHRINK,
			GTK_SHRINK, 5, 1);
      gtk_widget_set_tooltip_text (button,
				   _("Compile and apply this channel's formula..."));
      g_signal_connect (GTK_OBJECT (button), "clicked",
			G_CALLBACK (cb_editor_compile),
			GINT_TO_POINTER (i));
      g_signal_connect_swapped (GTK_OBJECT (button), "clicked",
				G_CALLBACK (gimp_preview_invalidate),
				(gpointer) dialog_preview);
    }

  hbox = gtk_hbox_new (FALSE, 5);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
  gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  button = gtk_button_new_from_stock (GTK_STOCK_NEW);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  gtk_widget_set_tooltip_text (button,
			       _("Create a new filter from scratch..."));
  g_signal_connect (GTK_OBJECT (button), "clicked",
		    G_CALLBACK (cb_editor_filter_new), NULL);
  g_signal_connect_swapped (GTK_OBJECT (button), "clicked",
			    G_CALLBACK (gimp_preview_invalidate),
			    (gpointer) dialog_preview);

  button = gtk_button_new_from_stock (GTK_STOCK_OPEN);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  gtk_widget_set_tooltip_text (button,
			       _("Load a filter file into the editor..."));
  g_signal_connect (GTK_OBJECT (button), "clicked",
		    G_CALLBACK (cb_editor_filter_load),
		    (gpointer) dialog_dialog);
  g_signal_connect_swapped (GTK_OBJECT (button), "clicked",
			    G_CALLBACK (gimp_preview_invalidate),
			    (gpointer) dialog_preview);

  button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  gtk_widget_set_tooltip_text (button, _("Save the current filter..."));
  g_signal_connect (GTK_OBJECT (button), "clicked",
		    G_CALLBACK (cb_editor_filter_save),
		    (gpointer) dialog_dialog);

  button = gtk_button_new_from_stock (GTK_STOCK_SAVE_AS);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  gtk_widget_set_tooltip_text (button,
			       _("Save the current filter as a new file..."));
  g_signal_connect (GTK_OBJECT (button), "clicked",
		    G_CALLBACK (cb_editor_filter_save_as),
		    (gpointer) dialog_dialog);

  gtk_widget_show_all (vbox);
}


/*********************************************************************
 *
 *  Filter Manager Panel
 */

static gboolean
cb_manager_append_filter (const UserFilter *  filter,
			  const gchar * const fpath,
			  gpointer            data)
{
  GtkTreeIter iter;
  gchar *     category;
  gchar *     title;
  gchar *     author;
  gchar *     copyright;

  category   = guf_filter_get_category (filter);
  title      = guf_filter_get_locale_title (filter);
  author     = guf_filter_get_author (filter);
  copyright  = guf_filter_get_copyright (filter);

  gtk_list_store_append (GTK_LIST_STORE(data), &iter);
  gtk_list_store_set (GTK_LIST_STORE(data), &iter,
		      COL_REGISTER,  guf_filter_get_registered (filter),
		      COL_CATEGORY,  category,
		      COL_TITLE,     title,
		      COL_AUTHOR,    author,
		      COL_COPYRIGHT, copyright,
		      COL_FNAME,     fpath,
		      -1);
  g_free (category);
  g_free (title);
  g_free (author);
  g_free (copyright);
  return TRUE;
}


static void
cb_manager_load_filter (const GtkWidget * widget,
			GtkTreeView *     filters)
{
  GtkTreeSelection * selection;
  GtkTreeModel *     store;
  GtkTreeIter        iter;

  if (filters &&
      (store = gtk_tree_view_get_model (filters)) &&
      (selection = gtk_tree_view_get_selection (filters)) &&
      gtk_tree_selection_get_selected (selection, &store, &iter))
    {
      gchar * fname;

      gtk_tree_model_get (store, &iter,
			  COL_FNAME, &fname, -1);
      if (fname)
	{
	  if (!guf_read (fname, GUF_PROTOCOL, filter))
	    {
	      /*
	       *  Failed to load a filter file, fallback to internal
	       *  default filter...
	       */
	      g_warning ("%s '%s'\n",
			 _("Unable to load filter"), fname);
	      filter = guf_filter_renew (filter);
	      g_free (last_filter);
	      g_free (fname);
	      last_filter = NULL;
	    }
	  else
	    {
	      g_free (last_filter);
	      last_filter = fname;
	    }
	  uf_update_internals (TRUE);
	}
    }
}


static void
cb_manager_filter_double_click (GtkTreeView *             view,
				const GtkTreePath *       path,
				const GtkTreeViewColumn * col,
				const gpointer            data)
{
  cb_manager_load_filter (NULL, view);
}


static void
cb_manager_filter_register (GimpCellRendererToggle * cell,
			    const gchar * const      path,
			    const GdkModifierType    state,
			    GtkTreeView *            filters)
{
  static gboolean notified = FALSE;
  GtkTreeModel *  store;
  GtkTreeIter     iter;

  if (filters &&
      (store = gtk_tree_view_get_model (filters)) &&
      gtk_tree_model_get_iter_from_string (store, &iter, path))
    {
      gchar *  fname = NULL;
      gboolean reg;

      gtk_tree_model_get (store, &iter,
			  COL_REGISTER, &reg,
			  COL_FNAME, &fname, -1);
      if (fname &&
	  guf_set_registered (fname, GUF_PROTOCOL, !reg))
	{
	  GtkTreePath * path;

	  gtk_list_store_set (GTK_LIST_STORE (store), &iter, COL_REGISTER, !reg, -1);
	  path = gtk_tree_model_get_path (store, &iter);
	  gtk_tree_model_row_changed (store, path, &iter);
	  gtk_tree_path_free (path);

	  if (!notified)
	    {
	      GtkWidget * dialog;
	      GtkWidget * box;

	      gimp_dialogs_show_help_button (FALSE);
	      dialog = gimp_dialog_new (_("Information"), plugin_name,
					NULL, 0,
					gimp_standard_help_func, NULL,
					GTK_STOCK_OK, GTK_RESPONSE_OK,
					NULL);
	      gimp_window_set_transient (GTK_WINDOW (dialog));
	      box = gimp_hint_box_new (_("Changes will become active the next time The GIMP is started."));
	      gtk_container_set_border_width (GTK_CONTAINER (box), 12);
	      gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), box);
	      gtk_widget_show (box);
	      gimp_dialog_run (GIMP_DIALOG (dialog));
	      gtk_widget_destroy (dialog);
	      notified = TRUE;
	    }
	}
      g_free (fname);
    }
}


static void
cb_manager_rescan_filters (const GtkWidget * widget,
			   GtkTreeView *     filters)
{
  GtkListStore * store;

  store = GTK_LIST_STORE (gtk_tree_view_get_model (filters));

  g_object_ref (store);
  gtk_tree_view_set_model (filters, NULL);

  gtk_list_store_clear (store);
  scan_filters (filter_path, cb_manager_append_filter, store);

  gtk_tree_view_set_model (filters, GTK_TREE_MODEL (store));
  g_object_unref (store);
}


static gint
cb_manager_sort_filters (GtkTreeModel * model,
			 GtkTreeIter *  a,
			 GtkTreeIter *  b,
			 const gpointer data)
{
  guint column = GPOINTER_TO_INT (data);
  gint  retval = 0;

  switch (column)
    {
    case COL_REGISTER:
      /*
       *  Sort by registered: registered, category, title
       */
      {
	gboolean ar;
	gboolean br;

	gtk_tree_model_get (model, a, column, &ar, -1);
	gtk_tree_model_get (model, b, column, &br, -1);

	if (ar != br)
	  {
	    retval = ar ? 1 : -1;
	  }
	if (retval == 0)
	  {
	    retval = cb_manager_sort_filters (model, a, b,
					      GINT_TO_POINTER (COL_CATEGORY));
	  }
      }
      break;
    case COL_CATEGORY:
      /*
       *  Sort by category:   category, title
       */
    case COL_TITLE:
      /*
       *  Sort by title:      title
       */
    case COL_AUTHOR:
      /*
       *  Sort by author:     author, title
       */
    case COL_COPYRIGHT:
      /*
       *  Sort by copyright:  copyright, category, title
       */
      {
	gchar * as;
	gchar * bs;

	gtk_tree_model_get (model, a, column, &as, -1);
	gtk_tree_model_get (model, b, column, &bs, -1);

	if (!as || !bs)
	  {
	    if (!as && !bs)
	      {
		break;
	      }
	    retval = !as ? -1 : 1;
	  }
	else
	  {
	    retval = g_utf8_collate (as, bs);
	  }
	g_free (as);
	g_free (bs);

	if (retval == 0)
	  {
	    switch (column)
	      {
	      case COL_CATEGORY:
	      case COL_AUTHOR:
		retval = cb_manager_sort_filters (model, a, b,
						  GINT_TO_POINTER (COL_TITLE));
		break;
	      case COL_COPYRIGHT:
		retval = cb_manager_sort_filters (model, a, b,
						  GINT_TO_POINTER (COL_CATEGORY));
		break;
	      default:
		break;
	      }
	  }
      }
      break;
    case COL_FNAME:
    default:
      break;
    }
  return retval;
}


static inline GtkTreeModel *
get_manager_filter_model (void)
{
  GtkListStore * store;

  store = gtk_list_store_new (MGR_NCOLS,
			      G_TYPE_BOOLEAN,	/* flag: registered in Gimp menu */
			      G_TYPE_STRING,	/* filter category */
			      G_TYPE_STRING,	/* filter name */
			      G_TYPE_STRING,	/* filter author */
			      G_TYPE_STRING,	/* filter copyright */
			      G_TYPE_STRING,	/* filter filename */
			      G_TYPE_LONG);	/* position of filter in file */
  scan_filters (filter_path, cb_manager_append_filter, store);
  return GTK_TREE_MODEL (store);
}


static inline GtkWidget *
get_manager_filter_view (void)
{
  GtkTreeViewColumn * column;
  GtkTreeSelection *  selection;
  GtkTreeSortable *   sortable;
  GtkCellRenderer *   renderer;
  GtkTreeModel *      model;
  GtkWidget *         view;

  model = get_manager_filter_model ();
  sortable = GTK_TREE_SORTABLE (model);
  gtk_tree_sortable_set_sort_func (sortable, COL_REGISTER,
				   cb_manager_sort_filters,
				   GINT_TO_POINTER (COL_REGISTER), NULL);
  gtk_tree_sortable_set_sort_func (sortable, COL_CATEGORY,
				   cb_manager_sort_filters,
				   GINT_TO_POINTER (COL_CATEGORY), NULL);
  gtk_tree_sortable_set_sort_func (sortable, COL_TITLE, cb_manager_sort_filters,
				   GINT_TO_POINTER (COL_TITLE), NULL);
  gtk_tree_sortable_set_sort_func (sortable, COL_AUTHOR, cb_manager_sort_filters,
				   GINT_TO_POINTER (COL_AUTHOR), NULL);
  gtk_tree_sortable_set_sort_func (sortable, COL_COPYRIGHT,
				   cb_manager_sort_filters,
				   GINT_TO_POINTER (COL_COPYRIGHT), NULL);
  gtk_tree_sortable_set_sort_column_id (sortable, COL_TITLE,
					GTK_SORT_ASCENDING);
  view = gtk_tree_view_new_with_model (model);
  g_object_unref (model);

  renderer = gimp_cell_renderer_toggle_new (GIMP_STOCK_PLUGIN);
  column = gtk_tree_view_column_new_with_attributes (_("Plug-in"),
						     renderer,
						     "active", COL_REGISTER,
						     NULL);
  g_signal_connect (GTK_OBJECT (renderer), "clicked",
		    G_CALLBACK (cb_manager_filter_register), view);
  gtk_tree_view_column_set_sort_column_id (column, COL_REGISTER);
  gtk_tree_view_insert_column (GTK_TREE_VIEW (view), column, -1);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Category"),
						     renderer,
						     "text", COL_CATEGORY,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COL_CATEGORY);
  gtk_tree_view_insert_column (GTK_TREE_VIEW (view), column, -1);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Title"),
						     renderer,
						     "text", COL_TITLE,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COL_TITLE);
  gtk_tree_view_insert_column (GTK_TREE_VIEW (view), column, -1);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Author"),
						     renderer,
						     "text", COL_AUTHOR,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COL_AUTHOR);
  gtk_tree_view_insert_column (GTK_TREE_VIEW (view), column, -1);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Copyright"),
						     renderer,
						     "text", COL_COPYRIGHT,
						     NULL);
  gtk_tree_view_column_set_sort_column_id (column, COL_COPYRIGHT);
  gtk_tree_view_insert_column (GTK_TREE_VIEW (view), column, -1);

  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
  gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);

  return view;
}


static inline GtkWidget *
uf_build_page_filter_manager (GtkWidget * notebook)
{
  GtkWidget * vbox;
  GtkWidget * box;
  GtkWidget * button;
  GtkWidget * scrolled_win;
  GtkWidget * filters;

  vbox = _uf_notebook_tab (notebook,
			   GIMP_STOCK_LIST,
			   _("Filter Manager"),
			   _("Manage your existing filters..."));

  box = gtk_vbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), box, TRUE, TRUE, 0);

  scrolled_win = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
				  GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
  gtk_container_add (GTK_CONTAINER (box), scrolled_win);

  filters = get_manager_filter_view ();
  gtk_container_add (GTK_CONTAINER (scrolled_win), filters);
  g_signal_connect (GTK_OBJECT (filters), "row-activated",
		    G_CALLBACK (cb_manager_filter_double_click), NULL);

  box = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_end (GTK_BOX (vbox), box, FALSE, FALSE, 0);

  button = gtk_button_new_from_stock (GTK_STOCK_REFRESH);
  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
  gtk_widget_set_tooltip_text (button,
			       _("Rescan the filter directories "
				 "for new filters..."));
  g_signal_connect (GTK_OBJECT (button), "clicked",
		    G_CALLBACK (cb_manager_rescan_filters), filters);

  button = gtk_button_new_from_stock (GTK_STOCK_OPEN);
  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
  gtk_widget_set_tooltip_text (button, _("Load the selected filter..."));
  g_signal_connect (GTK_OBJECT (button), "clicked",
		    G_CALLBACK (cb_manager_load_filter), filters);
  g_signal_connect_swapped (GTK_OBJECT (button), "clicked",
			    G_CALLBACK (gimp_preview_invalidate),
			    dialog_preview);

  gtk_widget_show_all (vbox);

  return filters;
}


/*********************************************************************
 *
 *  SearchPath Settings Panel
 */

static void
cb_settings_change_path (GimpPathEditor * editor,
			 GtkTreeView *    filters)
{
  g_free (filter_path);
  filter_path = gimp_path_editor_get_path (editor);
  gimp_gimprc_set (userfilter_path, filter_path);

  cb_manager_rescan_filters (NULL, filters);
}


static inline void
uf_build_page_filter_settings (GtkWidget * notebook,
			       GtkWidget * filters)
{
  GtkWidget * vbox;
  GtkWidget * editor;
  GtkWidget * label;

  vbox = _uf_notebook_tab (notebook,
			   GTK_STOCK_PREFERENCES,
			   _("Filter Directories"),
			   _("Set the paths where to look "
			     "for User Filter files..."));

  editor = gimp_path_editor_new (_("Directories with User Filter "
				   "files"), filter_path);
  gtk_container_set_border_width (GTK_CONTAINER (editor), 6);
  gtk_box_pack_start (GTK_BOX (vbox), editor, TRUE, TRUE, 0);
  g_signal_connect (GTK_OBJECT (editor), "path-changed",
		    G_CALLBACK (cb_settings_change_path), filters);

  label = gtk_label_new (_("Specify directories where to look for "
			   "User Filter files."));
  gtk_misc_set_padding (GTK_MISC (label), 6, 6);
  gtk_box_pack_end (GTK_BOX (vbox), label, FALSE, FALSE, 0);

  gtk_widget_show_all (vbox);
}


/*********************************************************************
 *
 *  Plug-in Information Panel
 */

static inline void
uf_build_page_plugin_info (GtkWidget * notebook)
{
  GtkWidget * vbox;
  GtkWidget * stuf;

  vbox = _uf_notebook_tab (notebook,
			   GTK_STOCK_ABOUT,
			   _("About User Filter"),
			   _("See who's behing this plugin..."));

  stuf = gtk_image_new_from_stock (GIMP_STOCK_WILBER, GTK_ICON_SIZE_DIALOG);
  gtk_box_pack_start (GTK_BOX (vbox), stuf, FALSE, FALSE, 20);

  stuf = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (stuf), "<big><b>User Filter v" PACKAGE_VERSION "</b></big>");
  gtk_box_pack_start (GTK_BOX (vbox), stuf, FALSE, FALSE, 20);

  stuf = gtk_label_new ("(C) 1997, 2003 Jens Ch. Restemeier");
  gtk_box_pack_start (GTK_BOX (vbox), stuf, FALSE, FALSE, 0);
  stuf = gtk_label_new ("(C) 2008 Torsten Neuer");
  gtk_box_pack_start (GTK_BOX (vbox), stuf, FALSE, FALSE, 0);

  stuf = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX (vbox), stuf, TRUE, TRUE, 0);

  stuf = gtk_label_new (_("User Filter is a generic filter tool that applies a user defined\n"
			  "formula on each color channel to modify an image."));
  gtk_box_pack_start (GTK_BOX (vbox), stuf, TRUE, TRUE, 0);

  stuf = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX (vbox), stuf, TRUE, TRUE, 0);

#if (PLUGIN_MINOR_VERSION % 2)

  stuf = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (vbox), stuf, FALSE, FALSE, 10);

  /* odd minor versions are considered as development versions */

  stuf = gtk_label_new (_("This is an unstable development release."));
  gimp_label_set_attributes (GTK_LABEL (stuf),
			     PANGO_ATTR_STYLE, PANGO_STYLE_ITALIC, -1);
  gtk_box_pack_end (GTK_BOX (vbox), stuf, FALSE, FALSE, 10);

#endif

  gtk_widget_show_all (vbox);
}


/*********************************************************************
 *
 *  User Filter IDE Dialog
 */

static inline GtkTreeView *
uf_build_notebook (GtkPaned * box)
{
  GtkWidget * filters;
  GtkWidget * notebook;

  notebook = gtk_notebook_new ();

  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
  gtk_paned_pack2 (box, notebook, TRUE, FALSE);
  _uf_build_page_filter_control (notebook);
  uf_build_page_filter_editor (notebook);
  filters = uf_build_page_filter_manager (notebook);
  uf_build_page_filter_settings (notebook, filters);
  uf_build_page_plugin_info (notebook);

  uf_update_internals (FALSE);

  gtk_notebook_popup_enable (GTK_NOTEBOOK (notebook));
  gtk_widget_show (notebook);

  return GTK_TREE_VIEW(filters);
}


gboolean
uf_dialog (GimpDrawable * drawable)
{
  GtkTreeView * filters;
  GtkWidget *   hbox;
  gboolean      run;

  gimp_ui_init (plugin_name, TRUE);

  dialog_dialog = gimp_dialog_new (_("User Filter"), plugin_name,
				   NULL, 0,
				   gimp_standard_help_func, plugin_name,
				   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				   GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog_dialog),
					   GTK_RESPONSE_OK,
					   GTK_RESPONSE_CANCEL, -1);
  gimp_window_set_transient (GTK_WINDOW (dialog_dialog));

  hbox = gtk_hpaned_new ();
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_dialog)->vbox), hbox, TRUE, TRUE, 0);
  dialog_preview = _uf_build_preview (GTK_PANED (hbox), drawable);
  filters = uf_build_notebook (GTK_PANED (hbox));
  gtk_widget_show (hbox);

  /*  Show the main containers  */

  gtk_widget_show (dialog_dialog);

  /*  Run the dialog  */

  run = (gimp_dialog_run (GIMP_DIALOG (dialog_dialog)) == GTK_RESPONSE_OK);

  /*  Clean up  */

  gtk_list_store_clear (GTK_LIST_STORE
			(gtk_tree_view_get_model (filters)));
  gtk_widget_destroy (dialog_dialog);

  return run;
}
