/*
 * (C) 2008  Torsten Neuer <tneuer@inwise.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: optimizer.h,v 1.4 2008/06/16 16:01:38 tneuer Exp $
 *
 *  $Log: optimizer.h,v $
 *  Revision 1.4  2008/06/16 16:01:38  tneuer
 *  Added code-only optimization to transpiler
 *
 *  Revision 1.3  2008/06/16 09:40:09  tneuer
 *  Added reduction of source function to channel source function for constant 'z' where 0 <= z <= 3
 *
 *  Revision 1.2  2008/05/21 12:30:39  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:05  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/19 07:49:50  tneuer
 *  Initial revision
 *
 */

#ifndef __OPTIMIZER_H__
#define __OPTIMIZER_H__

#include "scanner.h"
#include "userfilter.h"

/*
 *  Simple constant subexpession elimination.
 *
 *  This might not do much for a well written filter, especially
 *  not when previewing, since the only reducing of the code tree
 *  can occur on redundant code and computing constant values.
 *
 *  The only image dependant constants known during preview are
 *  M, X, Y, and Z, so normally not much of the tree is reduced
 *  when previewing (except when the formula is suboptimal).
 *
 *  When doing the final rendering, however, we can treat the values
 *  of the controls (and those that are directly derived from them)
 *  as constant values.  Running the optimizer here will then re-
 *  sult in an image processing that is up to 30% faster.
 *
 *  Parameters:
 *
 *  Doing "full" optimization also reduces the control values
 *  and should only be done before the final rendering.
 *
 *  The "code-only" optimization mutually excludes "full"
 *  optimization.  This is meant for (besides debugging)
 *  helping optimizing the formulas (by writing back the
 *  reduced tree as formula).
 */

void
optimize_code_tree (CodeTreePtr                     t,
		    const guint                     z,
		    const gboolean                  code_only,
		    const gboolean                  full,
		    const FilterEnvironment * const e);

#endif /* __OPTIMIZER_H__ */
