%{
/*
 * (C) 1997,2003  Jens Ch. Restemeier <jrestemeier@currantbun.com>
 *
 * (C) 2008       Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: parser.y,v 1.9 2008/06/16 09:36:13 tneuer Exp $
 *
 *  $Log: parser.y,v $
 *  Revision 1.9  2008/06/16 09:36:13  tneuer
 *  Added token for channel source functions (src[rgba])
 *
 *  Revision 1.8  2008/06/13 10:59:20  tneuer
 *  Added tokens t_get0 t_put0 for get () / put () optimization
 *
 *  Revision 1.7  2008/06/11 17:49:58  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.6  2008/06/10 14:09:28  tneuer
 *  Locale update
 *
 *  Revision 1.5  2008/06/10 11:49:15  tneuer
 *  Bugfix: Comma operator inside parenthesis gives syntax error
 *
 *  Revision 1.4  2008/06/07 13:21:14  tneuer
 *  Removed unhandled (and undocumented) function rst() from scanner and parser
 *
 *  Revision 1.3  2008/05/26 15:10:56  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.2  2008/05/21 12:30:39  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:04  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/01 16:06:09  tneuer
 *  Initial revision
 *
 */

#include "plugin-conf.h"

#include <glib/gmem.h>
#include <glib/gmessages.h>

#include "scanner.h"


#define YYSTYPE CodeTreePtr

static CodeTreePtr uf_root;


static inline gint
yyerror (const gchar * const s)
{
  g_message("%s.\n", s);
  return 0;
}


CodeTreePtr
get_code_tree (const gchar * const s)
{
  do_parse(s);
  return uf_root;
}


void
free_code_tree (CodeTreePtr a)
{
  int i;

  if ( (a == NULL) ||
       (a->op_type == v_int) )
    {
      return;
    }
  for ( i=a->value; i--; )
    {
      free_code_tree (a->nodes[i]);
    }
  g_free (a->nodes);
  g_free (a);
}

%}

%right '?' ':'

%right t_abs
%right t_add
%right t_cnv
%right t_ctl
%right t_dif
%right t_get
%right t_put
%right t_map
%right t_max
%right t_min
%right t_mix
%right t_rnd
%right t_scl
%right t_sqr
%right t_src
%right t_sub
%right t_val
%right t_c2d
%right t_c2m
%right t_cos
%right t_r2x
%right t_r2y
%right t_rad
%right t_sin
%right t_tan

%left '&' '|' '^'
%right t_and
%right t_or
%right t_eq
%right t_neq
%right t_shl
%right t_shr

%left '<' '>' t_leq t_geq
%left '+' '-'
%left '*' '/' '%'
%left t_comma ','
%token '(' ')'

%left t_sign '!' '~'

%token 'r' 'g' 'b' 'a' 'c' 'x' 'y' 'i' 'u' 'v' 'z' 'd' 'm'
%token t_rmin t_gmin t_bmin t_amin t_cmin t_imin t_umin t_vmin t_pmin t_tmin t_dmin t_mmin t_xmin t_ymin t_zmin
%token t_rmax t_gmax t_bmax t_amax t_cmax t_imax t_umax t_vmax t_pmax t_tmax t_dmax t_mmax t_xmax t_ymax t_zmax

%token v_int
%token t_error

%token t_get0 t_put0
%token t_srcr t_srcg t_srcb t_srca

%%

/****************************************************************************/

input:  { 
          uf_root = NULL; 
          YYABORT; 
	}
	| exp {
	  if ( depth == 0 )
	    {
	      uf_root = $1;
	    }
	  else
	    {
	      free_code_tree ($1);
	      if( depth < 0 ) 
		{
		  yyerror (_("Too many ')'" ));
		}
	      else
		{
		  yyerror (_("Missing ')'"));
		}
	      uf_root = NULL;
	    }
	  YYACCEPT;
	}
	| error { 
	  yyerrok;
	  uf_root = NULL; 
	  YYABORT; 
	}
	;


exp:    v_int { $$ = $1; }
	| 'r' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'r';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'g' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'g';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'b' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'b';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'a' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'a';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'c' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'c';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'x' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'x';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'y' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'y';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'z' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'z';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| t_xmax { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_xmax;
		$$->value = 0;
		$$->nodes = NULL;
	}
	| t_ymax { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_ymax;
		$$->value = 0;
		$$->nodes = NULL;
	}
	| t_zmax { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_zmax;
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'i' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'i';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'u' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'u';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'v' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'v';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'd' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'd';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| 'm' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = 'm';
		$$->value = 0;
		$$->nodes = NULL;
	}
	| t_mmax { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_mmax;
		$$->value = 0;
		$$->nodes = NULL;
	}
	| t_rmax {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 255;
		$$->nodes = NULL;
	}
	| t_gmax {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 255;
		$$->nodes = NULL;
	}
	| t_bmax {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 255;
		$$->nodes = NULL;
	}
	| t_amax {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 255;
		$$->nodes = NULL;
	}
	| t_imax {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 255;
		$$->nodes = NULL;
	}
	| t_cmax {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 255;
		$$->nodes = NULL;
	}
 	| t_rmin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_gmin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_bmin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_amin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_imin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_zmin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_cmin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
	| t_dmax { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 1024;
		$$->nodes = NULL;
	}
	| t_dmin { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
	| t_mmin { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_umax {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 56;
		$$->nodes = NULL;
	}
 	| t_vmax {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 78;
		$$->nodes = NULL;
	}
 	| t_umin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = -56;
		$$->nodes = NULL;
	}
 	| t_vmin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = -78;
		$$->nodes = NULL;
	}
 	| t_pmin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_tmin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_xmin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_ymin {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_pmax {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
 	| t_tmax {
		$$ = g_new (CodeTree, 1);
		$$->op_type = v_int;
		$$->value = 0;
		$$->nodes = NULL;
	}
        | exp '+' exp		{ 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '+';
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
        | exp '-' exp		{ 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '-';
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
        | exp '*' exp		{ 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '*';
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
        | exp '/' exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '/';
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	} 
        | exp '%' exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '%';
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	} 
        | exp '^' exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '^';
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	} 
        | exp '&' exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '&';
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	} 
        | exp '|' exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '|';
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	} 
        | '~' exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '~';
		$$->value = 1;
		$$->nodes = g_new (CodeTreePtr, 1);
		$$->nodes[0] = $2;
	} 
        | '+' exp %prec t_sign {
	        $$ = $2;
	}
        | '-' exp %prec t_sign {
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_sign;
		$$->value = 1;
		$$->nodes = g_new (CodeTreePtr, 1);
		$$->nodes[0] = $2;
	}         
        | exp '?' exp ':' exp   { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '?';
		$$->value = 3;
		$$->nodes = g_new (CodeTreePtr, 3);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
		$$->nodes[2] = $5;
	}
	| exp t_and exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_and;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
	| exp t_or exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_or;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
	| exp t_eq exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_eq;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
	| exp t_neq exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_neq;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
	| exp t_leq exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_leq;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
	| exp t_geq exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_geq;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
	| exp '<' exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '<';
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;	
		$$->nodes[1] = $3;
	}
	| exp '>' exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '>';
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
	| '!' exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = '!';
		$$->value = 1;
		$$->nodes = g_new (CodeTreePtr, 1);
		$$->nodes[0] = $2;
	}
	| exp t_shl exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_shl;
		$$->value = 1;
		$$->nodes = g_new (CodeTreePtr, 1);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
	| exp t_shr exp { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_shr;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
        | exp t_comma exp {
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_comma;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $1;
		$$->nodes[1] = $3;
	}
	| t_src '(' exp ',' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_src;
		$$->value = 3;
		$$->nodes = g_new (CodeTreePtr, 3);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
		$$->nodes[2] = $7;
	}
	| t_rad '(' exp ',' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_rad;
		$$->value = 3;
		$$->nodes = g_new (CodeTreePtr, 3);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
		$$->nodes[2] = $7;
	}
	| t_cnv '(' exp ',' exp ',' exp ',' exp ',' exp ',' exp ',' exp ',' exp ',' exp ',' exp  ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_cnv;
		$$->value = 10;
		$$->nodes = g_new (CodeTreePtr, 10);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
		$$->nodes[2] = $7;
		$$->nodes[3] = $9;
		$$->nodes[4] = $11;
		$$->nodes[5] = $13;
		$$->nodes[6] = $15;
		$$->nodes[7] = $17;
		$$->nodes[8] = $19;
		$$->nodes[9] = $21;
	}
	| t_ctl '(' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_ctl;
		$$->value = 1;
		$$->nodes = g_new (CodeTreePtr, 1);
		$$->nodes[0] = $3;
	}
	| t_abs '(' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_abs;
		$$->value = 1;
		$$->nodes = g_new (CodeTreePtr, 1);
		$$->nodes[0] = $3;
	}
	| t_dif '(' exp ',' exp  ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_dif;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
	}
	| t_scl '(' exp ',' exp ','exp ',' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_scl;
		$$->value = 5;
		$$->nodes = g_new (CodeTreePtr, 5);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
		$$->nodes[2] = $7;
		$$->nodes[3] = $9;
		$$->nodes[4] = $11;
	}
	| t_val '(' exp ',' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_val;
		$$->value = 3;
		$$->nodes = g_new (CodeTreePtr, 3);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
		$$->nodes[2] = $7;
	}
	| t_add '(' exp ',' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_add;
		$$->value = 3;
		$$->nodes = g_new (CodeTreePtr, 3);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
		$$->nodes[2] = $7;
	}
	| t_sub '(' exp ',' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_sub;
		$$->value = 3;
		$$->nodes = g_new (CodeTreePtr, 3);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
		$$->nodes[2] = $7;
	}
 	| t_mix '(' exp ',' exp ',' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_mix;
		$$->value = 4;
		$$->nodes = g_new (CodeTreePtr, 4);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
		$$->nodes[2] = $7;
		$$->nodes[3] = $9;
	}
	| t_rnd '(' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_rnd;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
	}
	| t_c2d '(' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_c2d;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
	}
	| t_c2m '(' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_c2m;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
	}
	| t_r2x '(' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_r2x;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
	}
	| t_r2y '(' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_r2y;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
	}
	| t_min '(' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_min;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
	}
	| t_max '(' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_max;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
	}
	| t_put '(' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_put;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
	}
	| t_map '(' exp ',' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_map;
		$$->value = 2;
		$$->nodes = g_new (CodeTreePtr, 2);
		$$->nodes[0] = $3;
		$$->nodes[1] = $5;
	}
	| t_get '(' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_get;
		$$->value = 1;
		$$->nodes = g_new (CodeTreePtr, 1);
		$$->nodes[0] = $3;
	}
        | t_sin '(' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_sin;
		$$->value = 1;
		$$->nodes = g_new (CodeTreePtr, 1);
		$$->nodes[0] = $3;
	}            
        | t_cos '(' exp ')' { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_cos;
		$$->value = 1;
		$$->nodes = g_new (CodeTreePtr, 1);
		$$->nodes[0] = $3;
	}            
        | t_tan '(' exp ')'    { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_tan;
		$$->value = 1;
		$$->nodes = g_new (CodeTreePtr, 1);
		$$->nodes[0] = $3;
	}            
        | t_sqr '(' exp ')'    { 
		$$ = g_new (CodeTree, 1);
		$$->op_type = t_sqr;
		$$->value = 1;
		$$->nodes = g_new (CodeTreePtr, 1);
		$$->nodes[0] = $3;
	}
        | '(' exp ')' 		{
	        $$ = $2;
	}
	| t_error { 
		$$ = NULL; 
		YYABORT; 
	}
	;

%%

