/*
 * (C) 2008  Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: registry.h,v 1.5 2008/06/17 15:41:30 tneuer Exp $
 *
 *  $Log: registry.h,v $
 *  Revision 1.5  2008/06/17 15:41:30  tneuer
 *  Minor code cleanups
 *
 *  Revision 1.4  2008/06/11 17:52:27  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.3  2008/05/16 14:02:56  tneuer
 *  Loading stand-alone (registered) filters.
 *
 *  Revision 1.2  2008/05/07 11:36:39  tneuer
 *  Registering filters for stand-alone operations.
 *
 *  Revision 1.1  2008/05/07 10:54:35  tneuer
 *  Added "userfilterrc" file for registering stand-alone filters.
 *
 */

#ifndef __REGISTRY_H__
#define __REGISTRY_H__

#include "guf.h"

extern const gchar * const registry_prefix;


/*
 *  We're hiding the GKeyFile behind a new class for
 *  type safety...
 */

typedef struct GKeyFile Registry;

#define registry_new() ((Registry *)g_key_file_new ())

#define registry_free(registry) g_key_file_free ((GKeyFile *)(registry))

Registry *
registry_renew (Registry * registry);


/*
 *  Load a registry file and replace the current
 *  registry entries (if any).  If the registry
 *  is not initialized, it will be initialized.
 */

Registry *
registry_load (Registry *          registry,
	       const gchar * const filename);


/*
 *  Register a filter in the registry.
 *
 *  Returns the PDB name (used as key value).
 *  The PDB name is a newly allocated (gchar *) that
 *  needs to be g_free ()'d again when not used anymore.
 */

gchar *
registry_register_filter (Registry *          registry,
			  const UserFilter *  filter,
			  const gchar * const fpath);


/*
 *  Query the filter path of a registered filter.
 *
 *  Returns a newly allocated string containing the
 *  path to the filter (or NULL if there is an error).
 */

gchar *
registry_get_filter_path (const Registry *    registry,
			  const gchar * const entry);


/*
 *  Save the registry file
 */

gboolean
registry_save (const Registry *    registry,
	       const gchar * const filename);

#endif /* __REGISTRY_H__ */
