%{
/*
 * (C) 1997,2003  Jens Ch. Restemeier <jrestemeier@currantbun.com>
 *
 * (C) 2008       Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: scanner.l,v 1.10 2008/06/11 17:49:59 tneuer Exp $
 *
 *  $Log: scanner.l,v $
 *  Revision 1.10  2008/06/11 17:49:59  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.9  2008/06/10 17:37:32  tneuer
 *  Bugfix: state stack underflow in function call with missing left paren
 *
 *  Revision 1.8  2008/06/10 17:11:21  tneuer
 *  Using scanner states for parenthesis modes
 *
 *  Revision 1.7  2008/06/10 14:09:29  tneuer
 *  Locale update
 *
 *  Revision 1.6  2008/06/10 11:49:15  tneuer
 *  Bugfix: Comma operator inside parenthesis gives syntax error
 *
 *  Revision 1.5  2008/06/07 13:21:17  tneuer
 *  Removed unhandled (and undocumented) function rst() from scanner and parser
 *
 *  Revision 1.4  2008/05/26 15:10:56  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.3  2008/05/21 16:11:21  tneuer
 *  Merged number scan rules (using strtol (buf, NULL, 0))
 *
 *  Revision 1.2  2008/05/21 15:56:13  tneuer
 *  Bugfix: Accept octal and sedecimal numbers in code now, not only decimal.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:05  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/01 16:06:36  tneuer
 *  Initial revision
 *
 */

#include "plugin-conf.h"

#include <string.h>
#include <glib.h>

#include "scanner.h"
#include "parser.h"

gint depth;

%}

OCTAL   "0"([0-7]+)
DECIMAL [0-9]+
HEX     "0x"([0-9a-zA-Z]+)

VALUE   {OCTAL}|{DECIMAL}|{HEX}

%option stack
%option nounput
%option nodefault
%option nostdinit
%option noyywrap
%option 7bit

%s FUNCTION ARGS PAREN

%%

"&&"	return t_and;
"||"	return t_or;
"=="	return t_eq;
"!="	return t_neq;
"<="	return t_leq;
">="	return t_geq;
"<<"	return t_shl;
">>"	return t_shr;

"rmin"	return t_rmin;
"gmin"	return t_gmin;
"bmin"	return t_bmin;
"amin"	return t_amin;
"cmin"	return t_cmin;
"imin"	return t_imin;
"umin"	return t_umin;
"vmin"	return t_vmin;
"pmin"	return t_pmin;
"tmin"	return t_tmin;
"dmin"	return t_dmin;
"mmin"	return t_mmin;
"xmin"	return t_xmin;
"ymin"	return t_ymin;
"zmin"	return t_zmin;

"R" |
"rmax"	return t_rmax;
"G" |
"gmax"	return t_gmax;
"B" |
"bmax"	return t_bmax;
"A" |
"amax"	return t_amax;
"C" |
"cmax"	return t_cmax;
"I" |
"imax"	return t_imax;

"U" |
"umax"	return t_umax;
"V" |
"vmax"	return t_vmax;

"pmax"	return t_pmax;
"tmax"	return t_tmax;

"D" |
"dmax"	return t_dmax;

"M" |
"mmax"	return t_mmax;

"X" |
"xmax"	return t_xmax;

"Y" |
"ymax"	return t_ymax;

"Z" |
"zmax"	return t_zmax;

"abs"	{ yy_push_state (FUNCTION); return t_abs; }
"add"	{ yy_push_state (FUNCTION); return t_add; }
"cnv"	{ yy_push_state (FUNCTION); return t_cnv; }
"ctl"	{ yy_push_state (FUNCTION); return t_ctl; }
"dif"	{ yy_push_state (FUNCTION); return t_dif; }
"get"	{ yy_push_state (FUNCTION); return t_get; }
"put"	{ yy_push_state (FUNCTION); return t_put; }
"map"	{ yy_push_state (FUNCTION); return t_map; }
"max"	{ yy_push_state (FUNCTION); return t_max; }
"min"	{ yy_push_state (FUNCTION); return t_min; }
"mix"	{ yy_push_state (FUNCTION); return t_mix; }
"rnd"	{ yy_push_state (FUNCTION); return t_rnd; }
"scl"	{ yy_push_state (FUNCTION); return t_scl; }
"sqr"	{ yy_push_state (FUNCTION); return t_sqr; }
"src"	{ yy_push_state (FUNCTION); return t_src; }
"sub"	{ yy_push_state (FUNCTION); return t_sub; }
"val"	{ yy_push_state (FUNCTION); return t_val; }
"c2d"	{ yy_push_state (FUNCTION); return t_c2d; }
"c2m"	{ yy_push_state (FUNCTION); return t_c2m; }
"cos"	{ yy_push_state (FUNCTION); return t_cos; }
"r2x"	{ yy_push_state (FUNCTION); return t_r2x; }
"r2y"	{ yy_push_state (FUNCTION); return t_r2y; }
"rad"	{ yy_push_state (FUNCTION); return t_rad; }
"sin"	{ yy_push_state (FUNCTION); return t_sin; }
"tan"	{ yy_push_state (FUNCTION); return t_tan; }

[\t\r\n" "]+	/* ignore NL/CR/TAB/SPACE */

{VALUE} {
	  yylval = g_new (CodeTree, 1);
	  yylval->op_type = v_int;
	  yylval->value = strtol (yytext, NULL, 0);
	  yylval->nodes = NULL;
	  return v_int; 
	}

<FUNCTION>"("	{
          depth++;
          BEGIN (ARGS);
          return '(';
        }

"("	{
          depth++;
          yy_push_state (PAREN);
          return '(';
        }

<ARGS,PAREN>")" {
          yy_pop_state ();
	  depth--;
          return ')';
        }

")"	{
	  depth--;
          return ')';
        }

<ARGS>"," {
          return ',';
        }

","	{
          return t_comma;
        }

.	return yytext[0];

%%

gint
do_parse (const gchar * const s)
{
  YY_BUFFER_STATE yy_buffer_state;
  gint            r;

  depth = 0;
  yy_buffer_state = yy_scan_string (s);
  r = yyparse ();
  yy_delete_buffer (yy_buffer_state);
  while (YYSTATE != INITIAL)
    {
      yy_pop_state ();
    }
  return r;
}
