/*
 * (C) 1997,2003  Jens Ch. Restemeier <jrestemeier@currantbun.com>
 *
 * (C) 2008       Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: txt-read.c,v 1.8 2008/06/16 16:34:44 tneuer Exp $
 *
 *  $Log: txt-read.c,v $
 *  Revision 1.8  2008/06/16 16:34:44  tneuer
 *  Added guf_filter_(get|set)_edge_mode () access functions
 *
 *  Revision 1.7  2008/06/11 17:52:36  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.6  2008/06/07 16:35:58  tneuer
 *  Using g_strerror () instead of strerror ()
 *
 *  Revision 1.5  2008/06/06 22:43:19  tneuer
 *  Add support for "Gimp::Date" entry in 8bf/afs/ffl/txt file import routines
 *
 *  Revision 1.4  2008/06/05 16:36:26  tneuer
 *  Setting default stepping values for controls (=1) on import
 *
 *  Revision 1.3  2008/06/05 15:23:59  tneuer
 *  Use g_fopen () for UTF-8 compatibility
 *
 *  Revision 1.2  2008/05/21 12:30:41  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:04  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/25 22:13:01  tneuer
 *  Initial revision
 *
 */

#include "plugin-conf.h"

#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include <glib/gstdio.h>

#include "userfilter.h"
#include "guess-year.h"
#include "guf.h"
#include "txt.h"


enum {
  LBL_SIZE = 256,
  SRC_SIZE = 1024,
  BUF_SIZE = 8192
};

typedef struct FILTER_DATA
{
  gchar category[LBL_SIZE];
  gchar title[LBL_SIZE];
  gchar author[LBL_SIZE];
  gchar copyright[LBL_SIZE];
  gchar filename[LBL_SIZE];
  gchar ctl[N_CONTROLS][LBL_SIZE];
  gint  val[N_CONTROLS];
  gchar src[N_CHANNELS][SRC_SIZE];
} FilterData;


/*
 * This loads source-files from Harald Heim's Plugin Commander.
 */

static gboolean
load_source_txt (FILE *       f,
		 FilterData * data)
{
  gchar buffer[BUF_SIZE];
  gchar tok[BUF_SIZE];
  gchar arg[BUF_SIZE];
  gint  c;
  gint  i;

  /*
   *  Rewind to beginning of input stream...
   */
  fseek (f, 0, SEEK_SET);

  /*
   *  Read key / value pairs...
   */
  while (!feof (f))
    {
      for (c = 0; (((i = fgetc (f)) != '\r') && (i != '\n') && (!feof (f)));)
	{
	  if (c == sizeof (buffer))
	    {
	      return FALSE;
	    }
	  buffer[c++] = i;
	  if (feof (f))
	    {
	      return FALSE;
	    }
	}
      if (c == sizeof (buffer))
	{
	  return FALSE;
	}
      buffer[c] = '\0';

      do
	{
	  i = fgetc (f);
	}
      while (((i == '\r') || (i == '\n')) && (!feof (f)));

      if (!feof (f))
	{
	  fseek (f, -1, SEEK_CUR);
	}

      i = strspn (buffer, "\t\r\n ");
      strcpy (tok, &(buffer[i]));
      i = strcspn (tok, " :");
      strcpy (buffer, &(tok[i]));
      tok[i] = '\0';

      i = strspn (buffer, " :");
      strcpy (arg, &(buffer[i]));

      if ((i = strcspn (arg, "\n\r")) > 0)
	{
	  arg[i] = '\0';
	}

      if (!strcmp (tok, "Category"))
	strncpy (data->category, arg, sizeof (data->category) - 1);
      if (!strcmp (tok, "Title"))
	strncpy (data->title, arg, sizeof (data->title) - 1);
      if (!strcmp (tok, "Copyright"))
	strncpy (data->copyright, arg, sizeof (data->copyright) - 1);
      if (!strcmp (tok, "Author"))
	strncpy (data->author, arg, sizeof (data->author) - 1);
      if (!strcmp (tok, "Filename"))
	strncpy (data->filename, arg, sizeof (data->filename) - 1);
      if (!strcmp (tok, "R"))
	strncpy (data->src[0], arg, sizeof (data->src[0]) - 1);
      if (!strcmp (tok, "G"))
	strncpy (data->src[1], arg, sizeof (data->src[1]) - 1);
      if (!strcmp (tok, "B"))
	strncpy (data->src[2], arg, sizeof (data->src[2]) - 1);
      if (!strcmp (tok, "A"))
	strncpy (data->src[3], arg, sizeof (data->src[3]) - 1);
      if (!strncmp (tok, "ctl", 3))
	{
	  i = strcspn (tok, "[");
	  i += strspn (&(tok[i]), " [");
	  strcpy (buffer, &(tok[i]));
	  i = strcspn (buffer, "]");
	  buffer[i] = '\0';
	  i = atoi (buffer);
	  strncpy (data->ctl[i], arg, sizeof (data->ctl[i]) - 1);
	}
      if (!strncmp (tok, "val", 3))
	{
	  i = strcspn (tok, "[");
	  i += strspn (&(tok[i]), " [");
	  strcpy (buffer, &(tok[i]));
	  i = strcspn (buffer, "]");
	  buffer[i] = '\0';
	  i = atoi (buffer);
	  data->val[i] = atoi (arg);
	}
    }
  return TRUE;
}


/*
 *  Import a PluginCommander source file.
 *
 *  All unset entries are preset with default values
 *  (like version information, tooltips, etc...).
 */

gboolean
import_txt (const gchar * const filename,
	    UserFilter *        filter)
{
  gboolean success = FALSE;
  gchar    buffer[8];
  FILE *   f;

  if ((f = g_fopen (filename, "r")) == NULL)
    {
      g_warning ("TXT %s '%s': %s\n",
		 _("Error loading"), filename, g_strerror(errno));
      return FALSE;
    }

  if ( (fread (buffer, sizeof (buffer), 1, f) == 1) &&
       (!strncmp (buffer, "Category", sizeof (buffer))) )
    {
      FilterData * data;

      data = g_new0 (FilterData, 1);
      if (load_source_txt (f, data))
	{
	  gint i;

	  /*
	   *  GUF Header
	   *
	   *  TXT imports are all level-1 protocol files by default since
	   *  they won't have anything fancy.
	   */
	  guf_filter_set_protocol (filter, GUF_LEVEL_1);
	  
	  /*
	   *  Common information...
	   */
	  guf_filter_set_imported_category (filter, g_strstrip(data->category));
	  guf_filter_set_imported_title (filter, g_strstrip(data->title));
	  guf_filter_set_imported_copyright (filter, g_strstrip(data->copyright));
	  guf_filter_set_imported_author (filter, g_strstrip(data->author));
	  
	  /*
	   *  Filter version information (User Filter extension)...
	   */
	  guf_filter_set_version_all (filter, 1, 0, 0);

	  /*
	   *  Filter Factory only information (for backward compatibility)...
	   */
	  guf_filter_set_imported_8bf_name (filter, g_strstrip(data->filename));

	  /*
	   *  Gimp only information...
	   */
	  guf_filter_set_registered (filter, FALSE);
	  guf_filter_set_description (filter, "Imported TXT filter.");
	  guf_filter_set_date (filter, guess_year (data->copyright, filename), 1, 1);
	  guf_filter_set_edge_mode (filter, GIMP_PIXEL_FETCHER_EDGE_SMEAR);

	  /*
	   *  Maps (.txt does not know them)...
	   */
	  for (i = 0; i < N_MAPS; i++)
	    {
	      guf_filter_set_map_enabled (filter, i, FALSE);
	    }

	  /*
	   *  Controllers...
	   */
	  for (i = 0; i < N_CONTROLS; i++)
	    {
	      g_strstrip (data->ctl[i]);
	      if (guf_filter_set_ctl_enabled (filter, i, (strlen (data->ctl[i]) > 0)))
		{
		  guf_filter_set_imported_ctl_label (filter, i, data->ctl[i]);
		}
	      guf_filter_set_ctl_preset (filter, i, data->val[i]);
	      guf_filter_set_ctl_step (filter, i, 1);
	    }
      
	  /*
	   *  Source code...
	   */
	  for (i = 0; i < N_CHANNELS; i++)
	    {
	      guf_filter_set_imported_src (filter, i, g_strstrip(data->src[i]));
	    }

	  success = TRUE;
	}
      g_free (data);
    }

  fclose (f);
  if (!success)
    {
      g_warning ("TXT %s '%s': %s\n",
		 _("Error loading"), filename, _("Invalid type of file."));
    }
  return success;
}
