/*
 * (C) 1997,2003  Jens Ch. Restemeier <jrestemeier@currantbun.com>
 *
 * (C) 2008       Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: txt-save.c,v 1.7 2008/06/11 17:52:38 tneuer Exp $
 *
 *  $Log: txt-save.c,v $
 *  Revision 1.7  2008/06/11 17:52:38  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.6  2008/06/07 17:21:28  tneuer
 *  Using g_fprintf () instead of fprintf () to avoid UTF-8 pitfalls
 *
 *  Revision 1.5  2008/06/07 16:35:58  tneuer
 *  Using g_strerror () instead of strerror ()
 *
 *  Revision 1.4  2008/06/05 15:24:03  tneuer
 *  Use g_fopen () for UTF-8 compatibility
 *
 *  Revision 1.3  2008/05/26 15:10:56  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.2  2008/05/21 12:30:41  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:04  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/24 17:07:47  tneuer
 *  Initial revision
 *
 */

#include "plugin-conf.h"

#include <errno.h>
#include <string.h>

#include <glib/gstdio.h>

#include "txt.h"
#include "guf.h"
#include "userfilter.h"


/*
 *  Export the internal filter representation to a
 *  PluginCommander source file.
 *
 *  Values unknown to PluginCommander are not saved
 *  (like version information, tooltips, etc...).
 */

static void
write_pair (FILE *              f,
	    const gchar * const key,
	    gchar *             val)
{
  g_fprintf (f, "%s: %s\n", key, val );
  g_free (val);
}


static inline void
write_ctl (FILE *     f,
	   const gint ctl,
	   gchar *    val)
{
  g_fprintf (f, "ctl[%d]: %s\n", ctl, val );
  g_free (val);
}


gboolean
export_txt (const gchar * const filename,
	    UserFilter *        filter)
{
  FILE * f;
  gint   i;

  if (guf_filter_get_protocol (filter) > GUF_LEVEL_1)
    {
      g_warning ("TXT %s '%s': %s\n",
		 _("Error saving"), filename, _("Protocol level too high."));
      return FALSE;
    }

  if ((f = g_fopen (filename, "w")) == NULL)
    {
      g_warning ("TXT %s '%s': %s\n",
		 _("Error saving"), filename, g_strerror (errno));
      return FALSE;
    }

  write_pair (f, "Category",  guf_filter_get_category (filter));
  write_pair (f, "Title",     guf_filter_get_title (filter));
  write_pair (f, "Copyright", guf_filter_get_copyright (filter));
  write_pair (f, "Author",    guf_filter_get_author (filter));
  write_pair (f, "Filename",  guf_filter_get_8bf_name (filter));

  fputc ('\n', f);

  for (i = 0; i < N_CHANNELS; i++ )
    {
      static const gchar * RGBA [N_CHANNELS] = {
	"R", "G", "B", "A"
      };

      write_pair (f, RGBA[i], guf_filter_get_src (filter, i));
    }

  fputc ('\n', f);

  for (i = 0; i < N_CONTROLS; i++)
    {
      write_ctl (f, i, guf_filter_get_ctl_label (filter, i));
    }

  fputc ('\n', f);

  for (i = 0; i < N_CONTROLS; i++)
    {
      g_fprintf (f, "val[%i]: %i\n", i, guf_filter_get_ctl_preset (filter, i));
    }

  fclose (f);

  return TRUE;
}
